@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,8 +17,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_DEBUG_H
-#define __WEECHAT_DEBUG_H 1
+#ifndef WEECHAT_DEBUG_H
+#define WEECHAT_DEBUG_H 1
 
 struct t_gui_window_tree;
 
@@ -29,6 +29,9 @@ extern void debug_hdata ();
 extern void debug_hooks ();
 extern void debug_infolists ();
 extern void debug_directories ();
+extern void debug_time_start (const char *name);
+extern void debug_time_end (int display);
 extern void debug_init ();
+extern void debug_end ();
 
-#endif /* __WEECHAT_DEBUG_H */
+#endif /* WEECHAT_DEBUG_H */
