@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,18 +17,18 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_HASHTABLE_H
-#define __WEECHAT_HASHTABLE_H 1
+#ifndef WEECHAT_HASHTABLE_H
+#define WEECHAT_HASHTABLE_H 1
 
 struct t_hashtable;
 struct t_infolist_item;
 
-typedef unsigned long (t_hashtable_hash_key)(struct t_hashtable *hashtable,
-                                             const void *key);
+typedef unsigned long long (t_hashtable_hash_key)(struct t_hashtable *hashtable,
+                                                  const void *key);
 typedef int (t_hashtable_keycmp)(struct t_hashtable *hashtable,
                                  const void *key1, const void *key2);
 typedef void (t_hashtable_free_key)(struct t_hashtable *hashtable,
-                                    void *key, const void *value);
+                                    void *key);
 typedef void (t_hashtable_free_value)(struct t_hashtable *hashtable,
                                       const void *key, void *value);
 typedef void (t_hashtable_map)(void *data,
@@ -40,7 +40,7 @@ typedef void (t_hashtable_map_string)(void *data,
 
 /*
  * Hashtable is a structure with an array "htable", each entry is a pointer
- * to a linked list, and it is read with hashed key (as unsigned long).
+ * to a linked list, and it is read with hashed key (as unsigned long long).
  * Keys with same hashed key are grouped in a linked list pointed by htable.
  * The htable is not sorted, the linked list is sorted.
  *
@@ -112,7 +112,7 @@ struct t_hashtable
                                        /* never asked)                      */
 };
 
-extern unsigned long hashtable_hash_key_djb2 (const char *string);
+extern unsigned long long hashtable_hash_key_djb2 (const char *string);
 extern struct t_hashtable *hashtable_new (int size,
                                           const char *type_keys,
                                           const char *type_values,
@@ -128,7 +128,7 @@ extern struct t_hashtable_item *hashtable_set (struct t_hashtable *hashtable,
                                                const void *value);
 extern struct t_hashtable_item *hashtable_get_item (struct t_hashtable *hashtable,
                                                     const void *key,
-                                                    unsigned long *hash);
+                                                    unsigned long long *hash);
 extern void *hashtable_get (struct t_hashtable *hashtable, const void *key);
 extern int hashtable_has_key (struct t_hashtable *hashtable, const void *key);
 extern void hashtable_map (struct t_hashtable *hashtable,
@@ -155,4 +155,4 @@ extern void hashtable_free (struct t_hashtable *hashtable);
 extern void hashtable_print_log (struct t_hashtable *hashtable,
                                  const char *name);
 
-#endif /* __WEECHAT_HASHTABLE_H */
+#endif /* WEECHAT_HASHTABLE_H */
