@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Simon Arlott
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -18,8 +18,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_HOOK_H
-#define __WEECHAT_HOOK_H 1
+#ifndef WEECHAT_HOOK_H
+#define WEECHAT_HOOK_H 1
 
 #include <unistd.h>
 
@@ -82,6 +82,14 @@ enum t_hook_type
 /* max calls that can be done for a command (recursive calls) */
 #define HOOK_COMMAND_MAX_CALLS  5
 
+/* return code when a command is executed */
+#define HOOK_COMMAND_EXEC_OK                    1
+#define HOOK_COMMAND_EXEC_ERROR                 0
+#define HOOK_COMMAND_EXEC_NOT_FOUND            -1
+#define HOOK_COMMAND_EXEC_AMBIGUOUS_PLUGINS    -2
+#define HOOK_COMMAND_EXEC_AMBIGUOUS_INCOMPLETE -3
+#define HOOK_COMMAND_EXEC_RUNNING              -4
+
 /* flags for fd hooks */
 #define HOOK_FD_FLAG_READ       1
 #define HOOK_FD_FLAG_WRITE      2
@@ -213,6 +221,7 @@ struct t_hook_process
     t_hook_callback_process *callback; /* process callback (after child end)*/
     char *command;                     /* command executed by child         */
     struct t_hashtable *options;       /* options for process (see doc)     */
+    int detached;                      /* detached mode (background)        */
     long timeout;                      /* timeout (ms) (0 = no timeout)     */
     int child_read[3];                 /* read stdin/out/err data from child*/
     int child_write[3];                /* write stdin/out/err data for child*/
@@ -221,6 +230,7 @@ struct t_hook_process
     struct t_hook *hook_timer;         /* timer to check if child has died  */
     char *buffer[3];                   /* buffers for child stdin/out/err   */
     int buffer_size[3];                /* size of child stdin/out/err       */
+    int buffer_flush;                  /* bytes to flush output buffers     */
 };
 
 /* hook connect */
@@ -235,13 +245,13 @@ typedef int (gnutls_callback_t)(void *data, gnutls_session_t tls_session,
                                 const gnutls_datum_t *req_ca, int nreq,
                                 const gnutls_pk_algorithm_t *pk_algos,
                                 int pk_algos_len,
-#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00
+#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00 /* 2.11.0 */
                                 gnutls_retr2_st *answer,
 #else
                                 gnutls_retr_st *answer,
-#endif
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x020b00 */
                                 int action);
-#endif
+#endif /* HAVE_GNUTLS */
 
 struct t_hook_connect
 {
@@ -257,7 +267,7 @@ struct t_hook_connect
     gnutls_callback_t *gnutls_cb;      /* GnuTLS callback during handshake  */
     int gnutls_dhkey_size;             /* Diffie Hellman Key Exchange size  */
     char *gnutls_priorities;           /* GnuTLS priorities                 */
-#endif
+#endif /* HAVE_GNUTLS */
     char *local_hostname;              /* force local hostname (optional)   */
     int child_read;                    /* to read data in pipe from child   */
     int child_write;                   /* to write data in pipe for child   */
@@ -273,7 +283,7 @@ struct t_hook_connect
 #ifdef HOOK_CONNECT_MAX_SOCKETS
     int sock_v4[HOOK_CONNECT_MAX_SOCKETS];  /* IPv4 sockets for connecting  */
     int sock_v6[HOOK_CONNECT_MAX_SOCKETS];  /* IPv6 sockets for connecting  */
-#endif
+#endif /* HOOK_CONNECT_MAX_SOCKETS */
 };
 
 /* hook print */
@@ -428,6 +438,8 @@ struct t_hook_focus
 extern char *hook_type_string[];
 extern struct t_hook *weechat_hooks[];
 extern struct t_hook *last_weechat_hook[];
+extern int hooks_count[];
+extern int hooks_count_total;
 
 /* hook functions */
 
@@ -455,17 +467,13 @@ extern struct t_hook *hook_timer (struct t_weechat_plugin *plugin,
                                   int max_calls,
                                   t_hook_callback_timer *callback,
                                   void *callback_data);
-extern void hook_timer_time_to_next (struct timeval *tv_timeout);
 extern void hook_timer_exec ();
 extern struct t_hook *hook_fd (struct t_weechat_plugin *plugin, int fd,
                                int flag_read, int flag_write,
                                int flag_exception,
                                t_hook_callback_fd *callback,
                                void *callback_data);
-extern int hook_fd_set (fd_set *read_fds, fd_set *write_fds,
-                        fd_set *exception_fds);
-extern void hook_fd_exec (fd_set *read_fds, fd_set *write_fds,
-                          fd_set *exception_fds);
+extern void hook_fd_exec ();
 extern struct t_hook *hook_process (struct t_weechat_plugin *plugin,
                                     const char *command,
                                     int timeout,
@@ -492,12 +500,12 @@ extern int hook_connect_gnutls_set_certificates (gnutls_session_t tls_session,
                                                  const gnutls_datum_t *req_ca, int nreq,
                                                  const gnutls_pk_algorithm_t *pk_algos,
                                                  int pk_algos_len,
-#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00
+#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00 /* 2.11.0 */
                                                  gnutls_retr2_st *answer);
 #else
                                                  gnutls_retr_st *answer);
-#endif
-#endif
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x020b00 */
+#endif /* HAVE_GNUTLS */
 extern struct t_hook *hook_print (struct t_weechat_plugin *plugin,
                                   struct t_gui_buffer *buffer,
                                   const char *tags, const char *message,
@@ -510,14 +518,14 @@ extern struct t_hook *hook_signal (struct t_weechat_plugin *plugin,
                                    const char *signal,
                                    t_hook_callback_signal *callback,
                                    void *callback_data);
-extern void hook_signal_send (const char *signal, const char *type_data,
-                              void *signal_data);
+extern int hook_signal_send (const char *signal, const char *type_data,
+                             void *signal_data);
 extern struct t_hook *hook_hsignal (struct t_weechat_plugin *plugin,
                                     const char *signal,
                                     t_hook_callback_hsignal *callback,
                                     void *callback_data);
-extern void hook_hsignal_send (const char *signal,
-                               struct t_hashtable *hashtable);
+extern int hook_hsignal_send (const char *signal,
+                              struct t_hashtable *hashtable);
 extern struct t_hook *hook_config (struct t_weechat_plugin *plugin,
                                    const char *option,
                                    t_hook_callback_config *callback,
@@ -598,4 +606,4 @@ extern int hook_add_to_infolist (struct t_infolist *infolist,
                                  const char *arguments);
 extern void hook_print_log ();
 
-#endif /* __WEECHAT_HOOK_H */
+#endif /* WEECHAT_HOOK_H */
