@@ -82,6 +82,7 @@
 int weechat_debug_core = 0;            /* debug level for core              */
 char *weechat_argv0 = NULL;            /* WeeChat binary file name (argv[0])*/
 int weechat_upgrading = 0;             /* =1 if WeeChat is upgrading        */
+int weechat_first_start = 0;           /* first start of WeeChat?           */
 time_t weechat_first_start_time = 0;   /* start time (used by /uptime cmd)  */
 int weechat_upgrade_count = 0;         /* number of /upgrade done           */
 struct timeval weechat_current_start_timeval; /* start time used to display */
@@ -89,6 +90,7 @@ struct timeval weechat_current_start_timeval; /* start time used to display */
 int weechat_quit = 0;                  /* = 1 if quit request from user     */
 int weechat_sigsegv = 0;               /* SIGSEGV received?                 */
 char *weechat_home = NULL;             /* home dir. (default: ~/.weechat)   */
+int weechat_locale_ok = 0;             /* is locale OK?                     */
 char *weechat_local_charset = NULL;    /* example: ISO-8859-1, UTF-8        */
 int weechat_server_cmd_line = 0;       /* at least 1 server on cmd line     */
 int weechat_auto_load_plugins = 1;     /* auto load plugins                 */
@@ -109,16 +111,17 @@ void
 weechat_display_copyright ()
 {
     string_iconv_fprintf (stdout, "\n");
-    string_iconv_fprintf (stdout,
-                          /* TRANSLATORS: "%s %s" after "compiled on" is date and time */
-                          _("WeeChat %s Copyright %s, compiled on %s %s\n"
-                            "Developed by Sébastien Helleu <flashcode@flashtux.org> "
-                            "- %s"),
-                          version_get_version_with_git (),
-                          WEECHAT_COPYRIGHT_DATE,
-                          version_get_compilation_date (),
-                          version_get_compilation_time (),
-                          WEECHAT_WEBSITE);
+    string_iconv_fprintf (
+        stdout,
+        /* TRANSLATORS: "%s %s" after "compiled on" is date and time */
+        _("WeeChat %s Copyright %s, compiled on %s %s\n"
+          "Developed by Sébastien Helleu <flashcode@flashtux.org> "
+          "- %s"),
+        version_get_version_with_git (),
+        WEECHAT_COPYRIGHT_DATE,
+        version_get_compilation_date (),
+        version_get_compilation_time (),
+        WEECHAT_WEBSITE);
     string_iconv_fprintf (stdout, "\n");
 }
 
@@ -135,19 +138,26 @@ weechat_display_usage (char *exec_name)
                           _("Usage: %s [option...] [plugin:option...]\n"),
                           exec_name, exec_name);
     string_iconv_fprintf (stdout, "\n");
-    string_iconv_fprintf (stdout,
-                          _("  -a, --no-connect         disable auto-connect to servers at startup\n"
-                            "  -c, --colors             display default colors in terminal\n"
-                            "  -d, --dir <path>         set WeeChat home directory (default: ~/.weechat)\n"
-                            "  -h, --help               display this help\n"
-                            "  -l, --license            display WeeChat license\n"
-                            "  -p, --no-plugin          don't load any plugin at startup\n"
-                            "  -r, --run-command <cmd>  run command(s) after startup\n"
-                            "                           (many commands can be separated by semicolons)\n"
-                            "  -s, --no-script          don't load any script at startup\n"
-                            "      --upgrade            upgrade WeeChat using session files (see /help upgrade in WeeChat)\n"
-                            "  -v, --version            display WeeChat version\n"
-                            "  plugin:option            option for plugin (see man weechat)\n"));
+    string_iconv_fprintf (
+        stdout,
+        _("  -a, --no-connect         disable auto-connect to servers at "
+          "startup\n"
+          "  -c, --colors             display default colors in terminal\n"
+          "  -d, --dir <path>         set WeeChat home directory "
+          "(default: ~/.weechat)\n"
+          "                           (environment variable WEECHAT_HOME is "
+          "read if this option is not given)\n"
+          "  -h, --help               display this help\n"
+          "  -l, --license            display WeeChat license\n"
+          "  -p, --no-plugin          don't load any plugin at startup\n"
+          "  -r, --run-command <cmd>  run command(s) after startup\n"
+          "                           (many commands can be separated by "
+          "semicolons)\n"
+          "  -s, --no-script          don't load any script at startup\n"
+          "      --upgrade            upgrade WeeChat using session files "
+          "(see /help upgrade in WeeChat)\n"
+          "  -v, --version            display WeeChat version\n"
+          "  plugin:option            option for plugin (see man weechat)\n"));
     string_iconv_fprintf(stdout, "\n");
 }
 
@@ -220,7 +230,8 @@ weechat_parse_args (int argc, char *argv[])
         {
             /*
              * Electric-fence is not working fine when gnutls loads
-             * certificates and Valgrind reports many memory errors with gnutls.
+             * certificates and Valgrind reports many memory errors with
+             * gnutls.
              * This option disables the init/deinit of gnutls,
              * it must NOT be used for other purposes!
              */
@@ -268,6 +279,52 @@ weechat_parse_args (int argc, char *argv[])
     }
 }
 
+/*
+ * Expands and assigns given path to "weechat_home".
+ */
+
+void
+weechat_set_home_path (char *home_path)
+{
+    char *ptr_home;
+    int dir_length;
+
+    if (home_path[0] == '~')
+    {
+        /* replace leading '~' by $HOME */
+        ptr_home = getenv ("HOME");
+        if (!ptr_home)
+        {
+            string_iconv_fprintf (stderr,
+                                  _("Error: unable to get HOME directory\n"));
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        dir_length = strlen (ptr_home) + strlen (home_path + 1) + 1;
+        weechat_home = malloc (dir_length);
+        if (weechat_home)
+        {
+            snprintf (weechat_home, dir_length,
+                      "%s%s", ptr_home, home_path + 1);
+        }
+    }
+    else
+    {
+        weechat_home = strdup (home_path);
+    }
+
+    if (!weechat_home)
+    {
+        string_iconv_fprintf (stderr,
+                              _("Error: not enough memory for home "
+                                "directory\n"));
+        weechat_shutdown (EXIT_FAILURE, 0);
+        /* make C static analyzer happy (never executed) */
+        return;
+    }
+}
+
 /*
  * Creates WeeChat home directory (by default ~/.weechat).
  *
@@ -278,13 +335,25 @@ weechat_parse_args (int argc, char *argv[])
 void
 weechat_create_home_dir ()
 {
-    char *ptr_home, *config_weechat_home = WEECHAT_HOME;
-    int dir_length;
+    char *ptr_weechat_home, *config_weechat_home;
     struct stat statinfo;
 
+    /*
+     * weechat_home is not set yet: look for environment variable
+     * "WEECHAT_HOME"
+     */
     if (!weechat_home)
     {
-        if (strlen (config_weechat_home) == 0)
+        ptr_weechat_home = getenv ("WEECHAT_HOME");
+        if (ptr_weechat_home && ptr_weechat_home[0])
+            weechat_set_home_path (ptr_weechat_home);
+    }
+
+    /* weechat_home is still not set: try to use compile time default */
+    if (!weechat_home)
+    {
+        config_weechat_home = WEECHAT_HOME;
+        if (!config_weechat_home[0])
         {
             string_iconv_fprintf (stderr,
                                   _("Error: WEECHAT_HOME is undefined, check "
@@ -293,41 +362,7 @@ weechat_create_home_dir ()
             /* make C static analyzer happy (never executed) */
             return;
         }
-
-        if (config_weechat_home[0] == '~')
-        {
-            /* replace leading '~' by $HOME */
-            ptr_home = getenv ("HOME");
-            if (!ptr_home)
-            {
-                string_iconv_fprintf (stderr,
-                                      _("Error: unable to get HOME directory\n"));
-                weechat_shutdown (EXIT_FAILURE, 0);
-                /* make C static analyzer happy (never executed) */
-                return;
-            }
-            dir_length = strlen (ptr_home) + strlen (config_weechat_home + 1) + 1;
-            weechat_home = malloc (dir_length);
-            if (weechat_home)
-            {
-                snprintf (weechat_home, dir_length,
-                          "%s%s", ptr_home, config_weechat_home + 1);
-            }
-        }
-        else
-        {
-            weechat_home = strdup (config_weechat_home);
-        }
-
-        if (!weechat_home)
-        {
-            string_iconv_fprintf (stderr,
-                                  _("Error: not enough memory for home "
-                                    "directory\n"));
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
+        weechat_set_home_path (config_weechat_home);
     }
 
     /* if home already exists, it has to be a directory */
@@ -339,6 +374,8 @@ weechat_create_home_dir ()
                                   _("Error: home (%s) is not a directory\n"),
                                   weechat_home);
             weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
         }
     }
 
@@ -349,29 +386,32 @@ weechat_create_home_dir ()
                               _("Error: cannot create directory \"%s\"\n"),
                               weechat_home);
         weechat_shutdown (EXIT_FAILURE, 0);
+        /* make C static analyzer happy (never executed) */
+        return;
     }
 }
 
 /*
- * Displays WeeChat welcome message.
+ * Displays WeeChat startup message.
  */
 
 void
-weechat_welcome_message ()
+weechat_startup_message ()
 {
     if (CONFIG_BOOLEAN(config_startup_display_logo))
     {
-        gui_chat_printf (NULL,
-                         "%s  ___       __         ______________        _____ \n"
-                         "%s  __ |     / /___________  ____/__  /_______ __  /_\n"
-                         "%s  __ | /| / /_  _ \\  _ \\  /    __  __ \\  __ `/  __/\n"
-                         "%s  __ |/ |/ / /  __/  __/ /___  _  / / / /_/ // /_  \n"
-                         "%s  ____/|__/  \\___/\\___/\\____/  /_/ /_/\\__,_/ \\__/  ",
-                         GUI_COLOR(GUI_COLOR_CHAT_NICK),
-                         GUI_COLOR(GUI_COLOR_CHAT_NICK),
-                         GUI_COLOR(GUI_COLOR_CHAT_NICK),
-                         GUI_COLOR(GUI_COLOR_CHAT_NICK),
-                         GUI_COLOR(GUI_COLOR_CHAT_NICK));
+        gui_chat_printf (
+            NULL,
+            "%s  ___       __         ______________        _____ \n"
+            "%s  __ |     / /___________  ____/__  /_______ __  /_\n"
+            "%s  __ | /| / /_  _ \\  _ \\  /    __  __ \\  __ `/  __/\n"
+            "%s  __ |/ |/ / /  __/  __/ /___  _  / / / /_/ // /_  \n"
+            "%s  ____/|__/  \\___/\\___/\\____/  /_/ /_/\\__,_/ \\__/  ",
+            GUI_COLOR(GUI_COLOR_CHAT_NICK),
+            GUI_COLOR(GUI_COLOR_CHAT_NICK),
+            GUI_COLOR(GUI_COLOR_CHAT_NICK),
+            GUI_COLOR(GUI_COLOR_CHAT_NICK),
+            GUI_COLOR(GUI_COLOR_CHAT_NICK));
     }
     if (CONFIG_BOOLEAN(config_startup_display_version))
     {
@@ -380,8 +420,34 @@ weechat_welcome_message ()
     if (CONFIG_BOOLEAN(config_startup_display_logo) ||
         CONFIG_BOOLEAN(config_startup_display_version))
     {
-        gui_chat_printf (NULL,
-                         "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -");
+        gui_chat_printf (
+            NULL,
+            "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -");
+    }
+
+    if (weechat_first_start)
+    {
+        /* message on first run (when weechat.conf is created) */
+        gui_chat_printf (NULL, "");
+        gui_chat_printf (
+            NULL,
+            _("Welcome to WeeChat!\n"
+              "\n"
+              "If you are discovering WeeChat, it is recommended to read at "
+              "least the quickstart guide, and the user's guide if you have "
+              "some time; they explain main WeeChat concepts.\n"
+              "All WeeChat docs are available at: https://weechat.org/doc\n"
+              "\n"
+              "Moreover, there is inline help with /help on all commands and "
+              "options (use Tab key to complete the name).\n"
+              "The command /iset (script iset.pl) can help to customize "
+              "WeeChat: /script install iset.pl\n"
+              "\n"
+              "You can add and connect to an IRC server with /server and "
+              "/connect commands (see /help server)."));
+        gui_chat_printf (NULL, "");
+        gui_chat_printf (NULL, "---");
+        gui_chat_printf (NULL, "");
     }
 }
 
@@ -428,6 +494,24 @@ weechat_term_check ()
     }
 }
 
+/*
+ * Displays warning about wrong locale ($LANG and $LC_*) if they are detected
+ * as wrong.
+ */
+
+void
+weechat_locale_check ()
+{
+    if (!weechat_locale_ok)
+    {
+        gui_chat_printf (
+            NULL,
+            _("%sWarning: cannot set the locale; make sure $LANG and $LC_* "
+              "variables are correct"),
+            gui_chat_prefix[GUI_CHAT_PREFIX_ERROR]);
+    }
+}
+
 /*
  * Callback for system signal SIGHUP: quits WeeChat.
  */
@@ -435,9 +519,16 @@ weechat_term_check ()
 void
 weechat_sighup ()
 {
-    log_printf (_("Signal %s received, exiting WeeChat..."), "SIGHUP");
-    (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
-    weechat_quit = 1;
+    int rc;
+
+    rc = hook_signal_send ("signal_sighup",
+                           WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    if (rc != WEECHAT_RC_OK_EAT)
+    {
+        log_printf (_("Signal %s received, exiting WeeChat..."), "SIGHUP");
+        (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+        weechat_quit = 1;
+    }
 }
 
 /*
@@ -447,9 +538,16 @@ weechat_sighup ()
 void
 weechat_sigquit ()
 {
-    log_printf (_("Signal %s received, exiting WeeChat..."), "SIGQUIT");
-    (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
-    weechat_quit = 1;
+    int rc;
+
+    rc = hook_signal_send ("signal_sigquit",
+                           WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    if (rc != WEECHAT_RC_OK_EAT)
+    {
+        log_printf (_("Signal %s received, exiting WeeChat..."), "SIGQUIT");
+        (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+        weechat_quit = 1;
+    }
 }
 
 /*
@@ -459,9 +557,16 @@ weechat_sigquit ()
 void
 weechat_sigterm ()
 {
-    log_printf (_("Signal %s received, exiting WeeChat..."), "SIGTERM");
-    (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
-    weechat_quit = 1;
+    int rc;
+
+    rc = hook_signal_send ("signal_sigterm",
+                           WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    if (rc != WEECHAT_RC_OK_EAT)
+    {
+        log_printf (_("Signal %s received, exiting WeeChat..."), "SIGTERM");
+        (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+        weechat_quit = 1;
+    }
 }
 
 /*
@@ -501,18 +606,18 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     weechat_first_start_time = time (NULL); /* initialize start time        */
     gettimeofday (&weechat_current_start_timeval, NULL);
 
-    setlocale (LC_ALL, "");             /* initialize gettext               */
+    weechat_locale_ok = (setlocale (LC_ALL, "") != NULL);   /* init gettext */
 #ifdef ENABLE_NLS
     bindtextdomain (PACKAGE, LOCALEDIR);
     bind_textdomain_codeset (PACKAGE, "UTF-8");
     textdomain (PACKAGE);
-#endif
+#endif /* ENABLE_NLS */
 
 #ifdef HAVE_LANGINFO_CODESET
     weechat_local_charset = strdup (nl_langinfo (CODESET));
 #else
     weechat_local_charset = strdup ("");
-#endif
+#endif /* HAVE_LANGINFO_CODESET */
     utf8_init ();
 
     /* catch signals */
@@ -553,9 +658,10 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
         upgrade_weechat_load ();        /* upgrade with session file        */
         weechat_upgrade_count++;        /* increase /upgrade count          */
     }
-    weechat_welcome_message ();         /* display WeeChat welcome message  */
+    weechat_startup_message ();         /* display WeeChat startup message  */
     gui_chat_print_lines_waiting_buffer (NULL); /* display lines waiting    */
-    weechat_term_check ();              /* warnings about $TERM (if wrong)  */
+    weechat_term_check ();              /* warning about wrong $TERM        */
+    weechat_locale_check ();            /* warning about wrong locale       */
     command_startup (0);                /* command executed before plugins  */
     plugin_init (weechat_auto_load_plugins, /* init plugin interface(s)     */
                  argc, argv);
