@@ -1,7 +1,7 @@
 /*
  * gui-curses-window.c - window display functions for Curses GUI
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -53,6 +53,7 @@
 #include "../gui-layout.h"
 #include "../gui-line.h"
 #include "../gui-main.h"
+#include "../gui-mouse.h"
 #include "../gui-nicklist.h"
 #include "gui-curses.h"
 
@@ -474,7 +475,8 @@ gui_window_set_custom_color_bg (WINDOW *window, int bg)
  */
 
 void
-gui_window_set_custom_color_fg_bg (WINDOW *window, int fg, int bg)
+gui_window_set_custom_color_fg_bg (WINDOW *window, int fg, int bg,
+                                   int reset_attributes)
 {
     int attributes;
 
@@ -502,7 +504,7 @@ gui_window_set_custom_color_fg_bg (WINDOW *window, int fg, int bg)
         }
         else if ((fg & GUI_COLOR_EXTENDED_MASK) < GUI_CURSES_NUM_WEECHAT_COLORS)
         {
-            if (!(fg & GUI_COLOR_EXTENDED_KEEPATTR_FLAG))
+            if (reset_attributes && !(fg & GUI_COLOR_EXTENDED_KEEPATTR_FLAG))
                 gui_window_remove_color_style (window, A_ALL_ATTR);
             attributes = gui_color_get_extended_attrs (fg) |
                 gui_weechat_colors[fg & GUI_COLOR_EXTENDED_MASK].attributes;
@@ -843,7 +845,7 @@ gui_window_string_apply_color_fg_bg (unsigned char **string, WINDOW *window)
     }
     if (window && (fg >= 0) && (bg >= 0))
     {
-        gui_window_set_custom_color_fg_bg (window, fg, bg);
+        gui_window_set_custom_color_fg_bg (window, fg, bg, 1);
     }
 
     *string = ptr_string;
@@ -1161,38 +1163,6 @@ gui_window_draw_separators (struct t_gui_window *window)
     }
 }
 
-/*
- * Redraws a buffer.
- */
-
-void
-gui_window_redraw_buffer (struct t_gui_buffer *buffer)
-{
-    if (!gui_init_ok)
-        return;
-
-    gui_chat_draw (buffer, 1);
-}
-
-/*
- * Redraws all buffers.
- */
-
-void
-gui_window_redraw_all_buffers ()
-{
-    struct t_gui_buffer *ptr_buffer;
-
-    if (!gui_init_ok)
-        return;
-
-    for (ptr_buffer = gui_buffers; ptr_buffer;
-         ptr_buffer = ptr_buffer->next_buffer)
-    {
-        gui_window_redraw_buffer (ptr_buffer);
-    }
-}
-
 /*
  * Switches to another buffer in a window.
  */
@@ -1249,11 +1219,14 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
     }
 
     window->buffer = buffer;
-    gui_buffer_add_value_num_displayed (buffer, 1);
+
+    gui_buffer_set_active_buffer (buffer);
+    gui_buffer_compute_num_displayed ();
 
     if (!weechat_upgrading && (old_buffer != buffer))
-        gui_hotlist_remove_buffer (buffer);
+        gui_hotlist_remove_buffer (buffer, 0);
 
+    /* remove unused bars and add missing bars in window */
     gui_bar_window_remove_unused_bars (window);
     gui_bar_window_add_missing_bars (window);
 
@@ -1293,8 +1266,6 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
         window->scroll->lines_after = 0;
     }
 
-    gui_buffer_set_active_buffer (buffer);
-
     for (ptr_bar_window = window->bar_windows; ptr_bar_window;
          ptr_bar_window = ptr_bar_window->next_bar_window)
     {
@@ -1313,8 +1284,8 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
 
     if (old_buffer != buffer)
     {
-        hook_signal_send ("buffer_switch",
-                          WEECHAT_HOOK_SIGNAL_POINTER, buffer);
+        (void) hook_signal_send ("buffer_switch",
+                                 WEECHAT_HOOK_SIGNAL_POINTER, buffer);
     }
 }
 
@@ -1334,6 +1305,8 @@ gui_window_switch (struct t_gui_window *window)
     old_window = gui_current_window;
 
     gui_current_window = window;
+
+    /* remove unused bars and add missing bars in window */
     changes = gui_bar_window_remove_unused_bars (old_window)
         || gui_bar_window_add_missing_bars (old_window);
     if (changes)
@@ -1351,8 +1324,8 @@ gui_window_switch (struct t_gui_window *window)
 
     gui_input_move_to_buffer (old_window->buffer, window->buffer);
 
-    hook_signal_send ("window_switch",
-                      WEECHAT_HOOK_SIGNAL_POINTER, gui_current_window);
+    (void) hook_signal_send ("window_switch",
+                             WEECHAT_HOOK_SIGNAL_POINTER, gui_current_window);
 }
 
 /*
@@ -1394,8 +1367,8 @@ gui_window_page_up (struct t_gui_window *window)
                 snprintf (scroll, sizeof (scroll), "-%d",
                           num_lines + 1);
                 gui_window_scroll (window, scroll);
-                hook_signal_send ("window_scrolled",
-                                  WEECHAT_HOOK_SIGNAL_POINTER, window);
+                (void) hook_signal_send ("window_scrolled",
+                                         WEECHAT_HOOK_SIGNAL_POINTER, window);
             }
             break;
         case GUI_BUFFER_NUM_TYPES:
@@ -1451,8 +1424,8 @@ gui_window_page_down (struct t_gui_window *window)
             snprintf (scroll, sizeof (scroll), "+%d",
                       num_lines + 1);
             gui_window_scroll (window, scroll);
-            hook_signal_send ("window_scrolled",
-                              WEECHAT_HOOK_SIGNAL_POINTER, window);
+            (void) hook_signal_send ("window_scrolled",
+                                     WEECHAT_HOOK_SIGNAL_POINTER, window);
             break;
         case GUI_BUFFER_NUM_TYPES:
             break;
@@ -1491,8 +1464,8 @@ gui_window_scroll_up (struct t_gui_window *window)
                 snprintf (scroll, sizeof (scroll), "-%d",
                           CONFIG_INTEGER(config_look_scroll_amount));
                 gui_window_scroll (window, scroll);
-                hook_signal_send ("window_scrolled",
-                                  WEECHAT_HOOK_SIGNAL_POINTER, window);
+                (void) hook_signal_send ("window_scrolled",
+                                         WEECHAT_HOOK_SIGNAL_POINTER, window);
             }
             break;
         case GUI_BUFFER_NUM_TYPES:
@@ -1542,8 +1515,8 @@ gui_window_scroll_down (struct t_gui_window *window)
             snprintf (scroll, sizeof (scroll), "+%d",
                       CONFIG_INTEGER(config_look_scroll_amount));
             gui_window_scroll (window, scroll);
-            hook_signal_send ("window_scrolled",
-                              WEECHAT_HOOK_SIGNAL_POINTER, window);
+            (void) hook_signal_send ("window_scrolled",
+                                     WEECHAT_HOOK_SIGNAL_POINTER, window);
             break;
         case GUI_BUFFER_NUM_TYPES:
             break;
@@ -1575,8 +1548,8 @@ gui_window_scroll_top (struct t_gui_window *window)
             {
                 window->scroll->start_line = NULL;
                 gui_buffer_ask_chat_refresh (window->buffer, 2);
-                hook_signal_send ("window_scrolled",
-                                  WEECHAT_HOOK_SIGNAL_POINTER, window);
+                (void) hook_signal_send ("window_scrolled",
+                                         WEECHAT_HOOK_SIGNAL_POINTER, window);
             }
             break;
         case GUI_BUFFER_NUM_TYPES:
@@ -1607,7 +1580,7 @@ gui_window_scroll_bottom (struct t_gui_window *window)
             window->scroll->start_line = NULL;
             if (window->buffer->lines->lines_count > window->win_chat_height)
             {
-                snprintf (scroll, sizeof (scroll), "-%d",
+                snprintf (scroll, sizeof (scroll), "--%d",
                           window->win_chat_height - 1);
                 gui_window_scroll (window, scroll);
             }
@@ -1615,8 +1588,8 @@ gui_window_scroll_bottom (struct t_gui_window *window)
             {
                 gui_buffer_ask_chat_refresh (window->buffer, 2);
             }
-            hook_signal_send ("window_scrolled",
-                              WEECHAT_HOOK_SIGNAL_POINTER, window);
+            (void) hook_signal_send ("window_scrolled",
+                                     WEECHAT_HOOK_SIGNAL_POINTER, window);
             break;
         case GUI_BUFFER_NUM_TYPES:
             break;
@@ -1742,9 +1715,14 @@ gui_window_refresh_windows ()
 
     old_current_window = gui_current_window;
 
+    /* remove unused bars and add missing root bars */
+    gui_bar_window_remove_unused_bars (NULL);
+    gui_bar_window_add_missing_bars (NULL);
+
     for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
     {
         if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_ROOT)
+            && ptr_bar->bar_window
             && !CONFIG_BOOLEAN(ptr_bar->options[GUI_BAR_OPTION_HIDDEN]))
         {
             gui_bar_window_calculate_pos_size (ptr_bar->bar_window, NULL);
@@ -2379,6 +2357,77 @@ gui_window_refresh_screen (int full_refresh)
     gui_window_refresh_windows ();
 }
 
+/*
+ * Callback for bare display timer.
+ */
+
+int
+gui_window_bare_display_timer_cb (void *data, int remaining_calls)
+{
+    /* make C compiler happy */
+    (void) data;
+
+    if (gui_window_bare_display)
+        gui_window_bare_display_toggle (NULL);
+
+    if (remaining_calls == 0)
+        gui_window_bare_display_timer = NULL;
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Toggles bare display.
+ */
+
+void
+gui_window_bare_display_toggle (const char *delay)
+{
+    long seconds;
+    char *error;
+
+    gui_window_bare_display ^= 1;
+
+    if (gui_window_bare_display)
+    {
+        /* temporarily disable ncurses */
+        endwin ();
+        if (gui_mouse_enabled)
+            gui_mouse_disable ();
+        if (delay)
+        {
+            error = NULL;
+            seconds = strtol (delay, &error, 10);
+            if (error && !error[0] && (seconds >= 0))
+            {
+                if (gui_window_bare_display_timer)
+                {
+                    unhook (gui_window_bare_display_timer);
+                    gui_window_bare_display_timer = NULL;
+                }
+                gui_window_bare_display_timer = hook_timer (
+                    NULL,
+                    seconds * 1000, 0, 1,
+                    &gui_window_bare_display_timer_cb, NULL);
+            }
+        }
+    }
+    else
+    {
+        /* come back to standard display (with ncurses) */
+        refresh ();
+        if (gui_window_bare_display_timer)
+        {
+            unhook (gui_window_bare_display_timer);
+            gui_window_bare_display_timer = NULL;
+        }
+        if (CONFIG_BOOLEAN(config_look_mouse))
+            gui_mouse_enable ();
+    }
+
+    gui_window_ask_refresh (2);
+}
+
 /*
  * Sets terminal title.
  *
@@ -2472,6 +2521,7 @@ gui_window_send_clipboard (const char *storage_unit, const char *text)
         fprintf (stderr, "\033]52;%s;%s\a",
                  (storage_unit) ? storage_unit : "",
                  text_base64);
+        fflush (stderr);
         free (text_base64);
     }
 }
@@ -2496,6 +2546,7 @@ gui_window_set_bracketed_paste_mode (int enable)
              (screen) ? "\033P" : "",
              (enable) ? "h" : "l",
              (screen) ? "\033\\" : "");
+    fflush (stderr);
 }
 
 /*
