@@ -1,7 +1,7 @@
 /*
  * gui-input.c - input functions (used by all GUI)
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -123,7 +123,14 @@ gui_input_replace_input (struct t_gui_buffer *buffer, const char *new_input)
 void
 gui_input_paste_pending_signal ()
 {
-    hook_signal_send ("input_paste_pending", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    if (CONFIG_BOOLEAN(config_look_bare_display_exit_on_input)
+        && gui_window_bare_display)
+    {
+        gui_window_bare_display_toggle (NULL);
+    }
+
+    (void) hook_signal_send ("input_paste_pending",
+                             WEECHAT_HOOK_SIGNAL_STRING, NULL);
 }
 
 /*
@@ -137,6 +144,12 @@ gui_input_text_changed_modifier_and_signal (struct t_gui_buffer *buffer,
 {
     char str_buffer[128], *new_input;
 
+    if (CONFIG_BOOLEAN(config_look_bare_display_exit_on_input)
+        && gui_window_bare_display)
+    {
+        gui_window_bare_display_toggle (NULL);
+    }
+
     if (!gui_cursor_mode)
     {
         if (save_undo)
@@ -166,7 +179,8 @@ gui_input_text_changed_modifier_and_signal (struct t_gui_buffer *buffer,
         gui_completion_stop (buffer->completion);
 
     /* send signal */
-    hook_signal_send ("input_text_changed", WEECHAT_HOOK_SIGNAL_POINTER, buffer);
+    (void) hook_signal_send ("input_text_changed",
+                             WEECHAT_HOOK_SIGNAL_POINTER, buffer);
 }
 
 /*
@@ -176,8 +190,14 @@ gui_input_text_changed_modifier_and_signal (struct t_gui_buffer *buffer,
 void
 gui_input_text_cursor_moved_signal (struct t_gui_buffer *buffer)
 {
-    hook_signal_send ("input_text_cursor_moved", WEECHAT_HOOK_SIGNAL_POINTER,
-                      buffer);
+    if (CONFIG_BOOLEAN(config_look_bare_display_exit_on_input)
+        && gui_window_bare_display)
+    {
+        gui_window_bare_display_toggle (NULL);
+    }
+
+    (void) hook_signal_send ("input_text_cursor_moved",
+                             WEECHAT_HOOK_SIGNAL_POINTER, buffer);
 }
 
 /*
@@ -187,7 +207,14 @@ gui_input_text_cursor_moved_signal (struct t_gui_buffer *buffer)
 void
 gui_input_search_signal (struct t_gui_buffer *buffer)
 {
-    hook_signal_send ("input_search", WEECHAT_HOOK_SIGNAL_POINTER, buffer);
+    if (CONFIG_BOOLEAN(config_look_bare_display_exit_on_input)
+        && gui_window_bare_display)
+    {
+        gui_window_bare_display_toggle (NULL);
+    }
+
+    (void) hook_signal_send ("input_search",
+                             WEECHAT_HOOK_SIGNAL_POINTER, buffer);
 }
 
 /*
@@ -243,11 +270,11 @@ gui_input_insert_string (struct t_gui_buffer *buffer, const char *string,
         buffer->input_buffer[buffer->input_buffer_size] = '\0';
 
         /* move end of string to the right */
-        ptr_start = utf8_add_offset (buffer->input_buffer, pos);
+        ptr_start = (char *)utf8_add_offset (buffer->input_buffer, pos);
         memmove (ptr_start + size, ptr_start, strlen (ptr_start));
 
         /* insert new string */
-        ptr_start = utf8_add_offset (buffer->input_buffer, pos);
+        ptr_start = (char *)utf8_add_offset (buffer->input_buffer, pos);
         strncpy (ptr_start, string_utf8, size);
 
         buffer->input_buffer_pos += length;
@@ -379,32 +406,34 @@ gui_input_clipboard_paste (struct t_gui_buffer *buffer)
 void
 gui_input_return (struct t_gui_buffer *buffer)
 {
-    struct t_gui_window *window;
     char *command;
 
-    window = gui_window_search_with_buffer (buffer);
-    if (window && window->buffer->input
-        && (window->buffer->input_buffer_size > 0))
+    if (CONFIG_BOOLEAN(config_look_bare_display_exit_on_input)
+        && gui_window_bare_display)
     {
-        window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-        command = strdup (window->buffer->input_buffer);
+        gui_window_bare_display_toggle (NULL);
+    }
+
+    if (buffer->input && (buffer->input_buffer_size > 0))
+    {
+        buffer->input_buffer[buffer->input_buffer_size] = '\0';
+        command = strdup (buffer->input_buffer);
         if (command)
         {
-            gui_history_add (window->buffer,
-                             window->buffer->input_buffer);
-            window->buffer->input_buffer[0] = '\0';
-            window->buffer->input_buffer_size = 0;
-            window->buffer->input_buffer_length = 0;
-            window->buffer->input_buffer_pos = 0;
-            window->buffer->input_buffer_1st_display = 0;
-            gui_buffer_undo_free_all (window->buffer);
-            window->buffer->ptr_history = NULL;
+            gui_history_add (buffer, buffer->input_buffer);
+            buffer->input_buffer[0] = '\0';
+            buffer->input_buffer_size = 0;
+            buffer->input_buffer_length = 0;
+            buffer->input_buffer_pos = 0;
+            buffer->input_buffer_1st_display = 0;
+            gui_buffer_undo_free_all (buffer);
+            buffer->ptr_history = NULL;
             gui_history_ptr = NULL;
-            gui_input_optimize_size (window->buffer);
-            gui_input_text_changed_modifier_and_signal (window->buffer,
+            gui_input_optimize_size (buffer);
+            gui_input_text_changed_modifier_and_signal (buffer,
                                                         0, /* save undo */
                                                         1); /* stop completion */
-            input_data (window->buffer, command);
+            (void) input_data (buffer, command);
             free (command);
         }
     }
@@ -538,7 +567,24 @@ gui_input_complete_previous (struct t_gui_buffer *buffer)
 }
 
 /*
- * Searches for text in buffer (default key: ctrl-R).
+ * Searches for text in buffer at current position (default key: ctrl-R).
+ */
+
+void
+gui_input_search_text_here (struct t_gui_buffer *buffer)
+{
+    struct t_gui_window *window;
+
+    window = gui_window_search_with_buffer (buffer);
+    if (window && (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED))
+    {
+        gui_window_search_start (window, window->scroll->start_line);
+        gui_input_search_signal (buffer);
+    }
+}
+
+/*
+ * Searches for text in buffer.
  */
 
 void
@@ -547,10 +593,9 @@ gui_input_search_text (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED))
     {
-        gui_window_search_start (window);
+        gui_window_search_start (window, NULL);
         gui_input_search_signal (buffer);
     }
 }
@@ -601,8 +646,7 @@ gui_input_search_switch_case (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
     {
         window->buffer->text_search_exact ^= 1;
         gui_window_search_restart (window);
@@ -621,8 +665,7 @@ gui_input_search_switch_regex (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
     {
         window->buffer->text_search_regex ^= 1;
         gui_window_search_restart (window);
@@ -640,9 +683,12 @@ gui_input_search_switch_where (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
     {
+        /* it's not possible to change that in a buffer with free content */
+        if (window->buffer->type == GUI_BUFFER_TYPE_FREE)
+            return;
+
         if (window->buffer->text_search_where == GUI_TEXT_SEARCH_IN_MESSAGE)
             window->buffer->text_search_where = GUI_TEXT_SEARCH_IN_PREFIX;
         else if (window->buffer->text_search_where == GUI_TEXT_SEARCH_IN_PREFIX)
@@ -664,8 +710,7 @@ gui_input_search_previous (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
     {
         window->buffer->text_search = GUI_TEXT_SEARCH_BACKWARD;
         (void) gui_window_search_text (window);
@@ -682,8 +727,7 @@ gui_input_search_next (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
     {
         window->buffer->text_search = GUI_TEXT_SEARCH_FORWARD;
         (void) gui_window_search_text (window);
@@ -691,7 +735,24 @@ gui_input_search_next (struct t_gui_buffer *buffer)
 }
 
 /*
- * Stops text search (default key: return during search).
+ * Stops text search at current position (default key: return during search).
+ */
+
+void
+gui_input_search_stop_here (struct t_gui_buffer *buffer)
+{
+    struct t_gui_window *window;
+
+    window = gui_window_search_with_buffer (buffer);
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    {
+        gui_window_search_stop_here (window);
+        gui_input_search_signal (buffer);
+    }
+}
+
+/*
+ * Stops text search (default key: ctrl-Q during search).
  */
 
 void
@@ -700,8 +761,7 @@ gui_input_search_stop (struct t_gui_buffer *buffer)
     struct t_gui_window *window;
 
     window = gui_window_search_with_buffer (buffer);
-    if (window && (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
-        && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
+    if (window && (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
     {
         gui_window_search_stop (window);
         gui_input_search_signal (buffer);
@@ -721,9 +781,9 @@ gui_input_delete_previous_char (struct t_gui_buffer *buffer)
     if (buffer->input && (buffer->input_buffer_pos > 0))
     {
         gui_buffer_undo_snap (buffer);
-        pos = utf8_add_offset (buffer->input_buffer,
-                               buffer->input_buffer_pos);
-        pos_last = utf8_prev_char (buffer->input_buffer, pos);
+        pos = (char *)utf8_add_offset (buffer->input_buffer,
+                                       buffer->input_buffer_pos);
+        pos_last = (char *)utf8_prev_char (buffer->input_buffer, pos);
         char_size = pos - pos_last;
         size_to_move = strlen (pos);
         memmove (pos_last, pos, size_to_move);
@@ -752,9 +812,9 @@ gui_input_delete_next_char (struct t_gui_buffer *buffer)
         && (buffer->input_buffer_pos < buffer->input_buffer_length))
     {
         gui_buffer_undo_snap (buffer);
-        pos = utf8_add_offset (buffer->input_buffer,
-                               buffer->input_buffer_pos);
-        pos_next = utf8_next_char (pos);
+        pos = (char *)utf8_add_offset (buffer->input_buffer,
+                                       buffer->input_buffer_pos);
+        pos_next = (char *)utf8_next_char (pos);
         char_size = pos_next - pos;
         size_to_move = strlen (pos_next);
         memmove (pos, pos_next, size_to_move);
@@ -781,30 +841,30 @@ gui_input_delete_previous_word (struct t_gui_buffer *buffer)
     if (buffer->input && (buffer->input_buffer_pos > 0))
     {
         gui_buffer_undo_snap (buffer);
-        start = utf8_add_offset (buffer->input_buffer,
-                                 buffer->input_buffer_pos - 1);
+        start = (char *)utf8_add_offset (buffer->input_buffer,
+                                         buffer->input_buffer_pos - 1);
         string = start;
-        while (string && (string[0] == ' '))
+        while (string && !string_is_word_char_input (string))
         {
-            string = utf8_prev_char (buffer->input_buffer, string);
+            string = (char *)utf8_prev_char (buffer->input_buffer, string);
         }
         if (string)
         {
-            while (string && (string[0] != ' '))
+            while (string && string_is_word_char_input (string))
             {
-                string = utf8_prev_char (buffer->input_buffer, string);
+                string = (char *)utf8_prev_char (buffer->input_buffer, string);
             }
             if (string)
             {
-                while (string && (string[0] == ' '))
+                while (string && !string_is_word_char_input (string))
                 {
-                    string = utf8_prev_char (buffer->input_buffer, string);
+                    string = (char *)utf8_prev_char (buffer->input_buffer, string);
                 }
             }
         }
 
         if (string)
-            string = utf8_next_char (utf8_next_char (string));
+            string = (char *)utf8_next_char (utf8_next_char (string));
         else
             string = buffer->input_buffer;
 
@@ -839,15 +899,15 @@ gui_input_delete_next_word (struct t_gui_buffer *buffer)
     if (buffer->input)
     {
         gui_buffer_undo_snap (buffer);
-        start = utf8_add_offset (buffer->input_buffer,
-                                 buffer->input_buffer_pos);
+        start = (char *)utf8_add_offset (buffer->input_buffer,
+                                         buffer->input_buffer_pos);
         string = start;
         length_deleted = 0;
         while (string[0])
         {
-            if ((string[0] == ' ') && (string > start))
+            if (!string_is_word_char_input (string) && (string > start))
                 break;
-            string = utf8_next_char (string);
+            string = (char *)utf8_next_char (string);
             length_deleted++;
         }
         size_deleted = string - start;
@@ -880,8 +940,8 @@ gui_input_delete_beginning_of_line (struct t_gui_buffer *buffer)
     if (buffer->input && (buffer->input_buffer_pos > 0))
     {
         gui_buffer_undo_snap (buffer);
-        start = utf8_add_offset (buffer->input_buffer,
-                                 buffer->input_buffer_pos);
+        start = (char *)utf8_add_offset (buffer->input_buffer,
+                                         buffer->input_buffer_pos);
         size_deleted = start - buffer->input_buffer;
         length_deleted = utf8_strnlen (buffer->input_buffer, size_deleted);
         gui_input_clipboard_copy (buffer->input_buffer,
@@ -913,8 +973,8 @@ gui_input_delete_end_of_line (struct t_gui_buffer *buffer)
     if (buffer->input)
     {
         gui_buffer_undo_snap (buffer);
-        start = utf8_add_offset (buffer->input_buffer,
-                                 buffer->input_buffer_pos);
+        start = (char *)utf8_add_offset (buffer->input_buffer,
+                                         buffer->input_buffer_pos);
         size_deleted = strlen (start);
         gui_input_clipboard_copy (start, size_deleted);
         start[0] = '\0';
@@ -966,9 +1026,9 @@ gui_input_transpose_chars (struct t_gui_buffer *buffer)
         if (buffer->input_buffer_pos == buffer->input_buffer_length)
             buffer->input_buffer_pos--;
 
-        start = utf8_add_offset (buffer->input_buffer,
-                                 buffer->input_buffer_pos);
-        prev_char = utf8_prev_char (buffer->input_buffer, start);
+        start = (char *)utf8_add_offset (buffer->input_buffer,
+                                         buffer->input_buffer_pos);
+        prev_char = (char *)utf8_prev_char (buffer->input_buffer, start);
         size_prev_char = start - prev_char;
         size_start_char = utf8_char_size (start);
 
@@ -1055,20 +1115,20 @@ gui_input_move_previous_word (struct t_gui_buffer *buffer)
     if (buffer->input
         && (buffer->input_buffer_pos > 0))
     {
-        pos = utf8_add_offset (buffer->input_buffer,
-                               buffer->input_buffer_pos - 1);
-        while (pos && (pos[0] == ' '))
+        pos = (char *)utf8_add_offset (buffer->input_buffer,
+                                       buffer->input_buffer_pos - 1);
+        while (pos && !string_is_word_char_input (pos))
         {
-            pos = utf8_prev_char (buffer->input_buffer, pos);
+            pos = (char *)utf8_prev_char (buffer->input_buffer, pos);
         }
         if (pos)
         {
-            while (pos && (pos[0] != ' '))
+            while (pos && string_is_word_char_input (pos))
             {
-                pos = utf8_prev_char (buffer->input_buffer, pos);
+                pos = (char *)utf8_prev_char (buffer->input_buffer, pos);
             }
             if (pos)
-                pos = utf8_next_char (pos);
+                pos = (char *)utf8_next_char (pos);
             else
                 pos = buffer->input_buffer;
             buffer->input_buffer_pos = utf8_pos (buffer->input_buffer,
@@ -1094,17 +1154,17 @@ gui_input_move_next_word (struct t_gui_buffer *buffer)
     if (buffer->input
         && (buffer->input_buffer_pos < buffer->input_buffer_length))
     {
-        pos = utf8_add_offset (buffer->input_buffer,
-                               buffer->input_buffer_pos);
-        while (pos[0] && (pos[0] == ' '))
+        pos = (char *)utf8_add_offset (buffer->input_buffer,
+                                       buffer->input_buffer_pos);
+        while (pos[0] && !string_is_word_char_input (pos))
         {
-            pos = utf8_next_char (pos);
+            pos = (char *)utf8_next_char (pos);
         }
         if (pos[0])
         {
-            while (pos[0] && (pos[0] != ' '))
+            while (pos[0] && string_is_word_char_input (pos))
             {
-                pos = utf8_next_char (pos);
+                pos = (char *)utf8_next_char (pos);
             }
             if (pos[0])
             {
@@ -1353,7 +1413,7 @@ gui_input_jump_smart (struct t_gui_buffer *buffer)
             if (!gui_hotlist_initial_buffer)
                 gui_hotlist_initial_buffer = window->buffer;
             gui_window_switch_to_buffer (window, gui_hotlist->buffer, 1);
-            gui_hotlist_remove_buffer (window->buffer);
+            gui_hotlist_remove_buffer (window->buffer, 0);
             scroll_to_bottom = 1;
         }
         else
@@ -1388,24 +1448,6 @@ gui_input_jump_smart (struct t_gui_buffer *buffer)
     }
 }
 
-/*
- * Jumps to last buffer (default key: meta-j, meta-l).
- */
-
-void
-gui_input_jump_last_buffer (struct t_gui_buffer *buffer)
-{
-    struct t_gui_window *window;
-
-    window = gui_window_search_with_buffer (buffer);
-    if (window
-        && (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED)
-        && last_gui_buffer)
-    {
-        gui_buffer_switch_by_number (window, last_gui_buffer->number);
-    }
-}
-
 /*
  * Jumps to last buffer displayed (before last jump to a buffer) (default key:
  * meta-/).
@@ -1566,7 +1608,7 @@ gui_input_switch_active_buffer (struct t_gui_buffer *buffer)
     struct t_gui_buffer *ptr_buffer;
     struct t_gui_window *window;
 
-    ptr_buffer = gui_buffer_get_next_active_buffer (buffer);
+    ptr_buffer = gui_buffer_get_next_active_buffer (buffer, 0);
     if (ptr_buffer)
     {
         gui_buffer_set_active_buffer (ptr_buffer);
@@ -1586,7 +1628,7 @@ gui_input_switch_active_buffer_previous (struct t_gui_buffer *buffer)
     struct t_gui_buffer *ptr_buffer;
     struct t_gui_window *window;
 
-    ptr_buffer = gui_buffer_get_previous_active_buffer (buffer);
+    ptr_buffer = gui_buffer_get_previous_active_buffer (buffer, 0);
     if (ptr_buffer)
     {
         gui_buffer_set_active_buffer (ptr_buffer);
@@ -1605,6 +1647,7 @@ void
 gui_input_zoom_merged_buffer (struct t_gui_buffer *buffer)
 {
     struct t_gui_window *ptr_window;
+    struct t_gui_buffer *ptr_buffer;
     int buffer_was_zoomed;
 
     /* do nothing if current buffer is not merged with another buffer */
@@ -1648,10 +1691,25 @@ gui_input_zoom_merged_buffer (struct t_gui_buffer *buffer)
         buffer->lines = buffer->mixed_lines;
     }
 
+    /* set "zoomed" in merged buffers */
+    for (ptr_buffer = gui_buffers; ptr_buffer;
+         ptr_buffer = ptr_buffer->next_buffer)
+    {
+        if (ptr_buffer->number > buffer->number)
+            break;
+        if (ptr_buffer->number == buffer->number)
+        {
+            ptr_buffer->zoomed = (buffer->active == 2) ? 1 : 0;
+        }
+    }
+
+    gui_buffer_compute_num_displayed ();
+
     gui_buffer_ask_chat_refresh (buffer, 2);
 
-    hook_signal_send ((buffer_was_zoomed) ? "buffer_unzoomed" : "buffer_zoomed",
-                      WEECHAT_HOOK_SIGNAL_POINTER, buffer);
+    (void) hook_signal_send ((buffer_was_zoomed) ?
+                             "buffer_unzoomed" : "buffer_zoomed",
+                             WEECHAT_HOOK_SIGNAL_POINTER, buffer);
 }
 
 /*
