@@ -1,7 +1,7 @@
 /*
  * gui-key.c - keyboard functions (used by all GUI)
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -31,6 +31,7 @@
 
 #include "../core/weechat.h"
 #include "../core/wee-config.h"
+#include "../core/wee-eval.h"
 #include "../core/wee-hashtable.h"
 #include "../core/wee-hdata.h"
 #include "../core/wee-hook.h"
@@ -176,7 +177,7 @@ gui_key_grab_init (int grab_command, const char *delay)
     gui_key_grab_count = 0;
     gui_key_grab_command = grab_command;
 
-    gui_key_grab_delay = GUI_KEY_GRAB_DELAY_DEFAULT;
+    gui_key_grab_delay = CONFIG_INTEGER(config_look_key_grab_delay);
     if (delay != NULL)
     {
         error = NULL;
@@ -632,59 +633,48 @@ gui_key_new (struct t_gui_buffer *buffer, int context, const char *key,
     if (!key || !command)
         return NULL;
 
-    if ((new_key = malloc (sizeof (*new_key))))
+    new_key = malloc (sizeof (*new_key));
+    if (!new_key)
+        return NULL;
+
+    new_key->key = gui_key_get_internal_code (key);
+    if (!new_key->key)
+        new_key->key = strdup (key);
+    new_key->command = strdup (command);
+    gui_key_set_areas (new_key);
+    gui_key_set_score (new_key);
+
+    if (buffer)
     {
-        new_key->key = gui_key_get_internal_code (key);
-        if (!new_key->key)
-            new_key->key = strdup (key);
-        if (!new_key->key)
-        {
-            free (new_key);
-            return NULL;
-        }
-        new_key->command = strdup (command);
-        if (!new_key->command)
-        {
-            free (new_key->key);
-            free (new_key);
-            return NULL;
-        }
-        gui_key_set_areas (new_key);
-        gui_key_set_score (new_key);
-
-        if (buffer)
-        {
-            gui_key_insert_sorted (&buffer->keys, &buffer->last_key,
-                                   &buffer->keys_count, new_key);
-        }
-        else
-        {
-            gui_key_insert_sorted (&gui_keys[context],
-                                   &last_gui_key[context],
-                                   &gui_keys_count[context], new_key);
-        }
-
-        expanded_name = gui_key_get_expanded_name (new_key->key);
-
-        hook_signal_send ("key_bind",
-                          WEECHAT_HOOK_SIGNAL_STRING, expanded_name);
-
-        if (gui_key_verbose)
-        {
-            gui_chat_printf (NULL,
-                             _("New key binding (context \"%s\"): "
-                               "%s%s => %s%s"),
-                             gui_key_context_string[context],
-                             (expanded_name) ? expanded_name : new_key->key,
-                             GUI_COLOR(GUI_COLOR_CHAT_DELIMITERS),
-                             GUI_COLOR(GUI_COLOR_CHAT),
-                             new_key->command);
-        }
-        if (expanded_name)
-                free (expanded_name);
+        gui_key_insert_sorted (&buffer->keys, &buffer->last_key,
+                               &buffer->keys_count, new_key);
     }
     else
-        return NULL;
+    {
+        gui_key_insert_sorted (&gui_keys[context],
+                               &last_gui_key[context],
+                               &gui_keys_count[context], new_key);
+    }
+
+    expanded_name = gui_key_get_expanded_name (new_key->key);
+
+    (void) hook_signal_send ("key_bind",
+                             WEECHAT_HOOK_SIGNAL_STRING, expanded_name);
+
+    if (gui_key_verbose)
+    {
+        gui_chat_printf (NULL,
+                         _("New key binding (context \"%s\"): "
+                           "%s%s => %s%s"),
+                         gui_key_context_string[context],
+                         (expanded_name) ? expanded_name : new_key->key,
+                         GUI_COLOR(GUI_COLOR_CHAT_DELIMITERS),
+                         GUI_COLOR(GUI_COLOR_CHAT),
+                         new_key->command);
+    }
+
+    if (expanded_name)
+        free (expanded_name);
 
     return new_key;
 }
@@ -886,8 +876,8 @@ gui_key_unbind (struct t_gui_buffer *buffer, int context, const char *key)
             gui_key_free (&gui_keys[context], &last_gui_key[context],
                           &gui_keys_count[context], ptr_key);
         }
-        hook_signal_send ("key_unbind",
-                          WEECHAT_HOOK_SIGNAL_STRING, (char *)key);
+        (void) hook_signal_send ("key_unbind",
+                                 WEECHAT_HOOK_SIGNAL_STRING, (char *)key);
         return 1;
     }
 
@@ -1040,7 +1030,7 @@ gui_key_focus_command (const char *key, int context,
                        struct t_hashtable **hashtable_focus)
 {
     struct t_gui_key *ptr_key;
-    int i, errors, matching, debug, rc;
+    int i, matching, debug, rc;
     long unsigned int value;
     char *command, **commands;
     const char *str_buffer;
@@ -1119,34 +1109,32 @@ gui_key_focus_command (const char *key, int context,
             gui_chat_printf (NULL, _("Command for key: \"%s\""),
                              ptr_key->command);
         }
-        commands = string_split_command (ptr_key->command, ';');
-        if (commands)
+        if (ptr_key->command)
         {
-            for (i = 0; commands[i]; i++)
+            commands = string_split_command (ptr_key->command, ';');
+            if (commands)
             {
-                if (string_strncasecmp (commands[i], "hsignal:", 8) == 0)
+                for (i = 0; commands[i]; i++)
                 {
-                    if (commands[i][8])
+                    if (string_strncasecmp (commands[i], "hsignal:", 8) == 0)
                     {
-                        if (debug)
+                        if (commands[i][8])
                         {
-                            gui_chat_printf (NULL,
-                                             _("Sending hsignal: \"%s\""),
-                                             commands[i] + 8);
+                            if (debug)
+                            {
+                                gui_chat_printf (NULL,
+                                                 _("Sending hsignal: \"%s\""),
+                                                 commands[i] + 8);
+                            }
+                            (void) hook_hsignal_send (commands[i] + 8,
+                                                      hashtable);
                         }
-                        hook_hsignal_send (commands[i] + 8, hashtable);
                     }
-                }
-                else
-                {
-                    command = string_replace_with_callback (commands[i],
-                                                            "${", "}",
-                                                            &gui_key_focus_command_replace_cb,
-                                                            hashtable,
-                                                            &errors);
-                    if (command)
+                    else
                     {
-                        if (errors == 0)
+                        command = eval_expression (commands[i], NULL,
+                                                   hashtable, NULL);
+                        if (command)
                         {
                             if (debug)
                             {
@@ -1156,13 +1144,13 @@ gui_key_focus_command (const char *key, int context,
                                                  command,
                                                  ptr_buffer->full_name);
                             }
-                            input_data (ptr_buffer, command);
+                            (void) input_data (ptr_buffer, command);
+                            free (command);
                         }
-                        free (command);
                     }
                 }
+                string_free_split (commands);
             }
-            string_free_split_command (commands);
         }
         hashtable_free (hashtable);
         return 1;
@@ -1246,6 +1234,34 @@ end:
     return rc;
 }
 
+/*
+ * Checks if the given combo of keys is complete or not.
+ * It's not complete if the end of string is in the middle of a key
+ * (for example meta- without the following char/key).
+ *
+ * Returns:
+ *   0: key is incomplete (truncated)
+ *   1: key is complete
+ */
+
+int
+gui_key_is_complete (const char *key)
+{
+    int length;
+
+    if (!key || !key[0])
+        return 1;
+
+    length = strlen (key);
+
+    if (((length >= 1) && (strcmp (key + length - 1, "\x01") == 0))
+        || ((length >= 2) && (strcmp (key + length - 2, "\x01[") == 0))
+        || ((length >= 3) && (strcmp (key + length - 3, "\x01[[") == 0)))
+        return 0;
+
+    return 1;
+}
+
 /*
  * Processes a new key pressed.
  *
@@ -1257,9 +1273,11 @@ end:
 int
 gui_key_pressed (const char *key_str)
 {
-    int i, first_key, context, length, length_key;
+    int i, first_key, context, length, length_key, rc, signal_sent;
     struct t_gui_key *ptr_key;
-    char **commands, *pos;
+    char *pos, signal_name[128], **commands;
+
+    signal_sent = 0;
 
     /* add key to buffer */
     first_key = (gui_key_combo_buffer[0] == '\0');
@@ -1287,13 +1305,14 @@ gui_key_pressed (const char *key_str)
         if (pos)
         {
             pos[0] = '\0';
-            gui_mouse_event_end ();
+            if (!gui_window_bare_display)
+                gui_mouse_event_end ();
             gui_mouse_event_init ();
         }
         return 0;
     }
 
-    if (strcmp (gui_key_combo_buffer, "\x01[[M") == 0)
+    if (strstr (gui_key_combo_buffer, "\x01[[M"))
     {
         gui_key_combo_buffer[0] = '\0';
         gui_mouse_event_init ();
@@ -1340,17 +1359,23 @@ gui_key_pressed (const char *key_str)
         if (strcmp (ptr_key->key, gui_key_combo_buffer) == 0)
         {
             /* exact combo found => execute command */
+            snprintf (signal_name, sizeof (signal_name),
+                      "key_combo_%s", gui_key_context_string[context]);
+            rc = hook_signal_send (signal_name,
+                                   WEECHAT_HOOK_SIGNAL_STRING,
+                                   gui_key_combo_buffer);
             gui_key_combo_buffer[0] = '\0';
-            if (ptr_key->command)
+            if ((rc != WEECHAT_RC_OK_EAT) && ptr_key->command)
             {
                 commands = string_split_command (ptr_key->command, ';');
                 if (commands)
                 {
                     for (i = 0; commands[i]; i++)
                     {
-                        input_data (gui_current_window->buffer, commands[i]);
+                        (void) input_data (gui_current_window->buffer,
+                                           commands[i]);
                     }
-                    string_free_split_command (commands);
+                    string_free_split (commands);
                 }
             }
         }
@@ -1358,6 +1383,16 @@ gui_key_pressed (const char *key_str)
     }
     else if (context == GUI_KEY_CONTEXT_CURSOR)
     {
+        signal_sent = 1;
+        snprintf (signal_name, sizeof (signal_name),
+                  "key_combo_%s", gui_key_context_string[context]);
+        if (hook_signal_send (signal_name,
+                              WEECHAT_HOOK_SIGNAL_STRING,
+                              gui_key_combo_buffer) == WEECHAT_RC_OK_EAT)
+        {
+            gui_key_combo_buffer[0] = '\0';
+            return 0;
+        }
         if (gui_key_focus (gui_key_combo_buffer, GUI_KEY_CONTEXT_CURSOR))
         {
             gui_key_combo_buffer[0] = '\0';
@@ -1365,7 +1400,21 @@ gui_key_pressed (const char *key_str)
         }
     }
 
-    gui_key_combo_buffer[0] = '\0';
+    if (!signal_sent)
+    {
+        snprintf (signal_name, sizeof (signal_name),
+                  "key_combo_%s", gui_key_context_string[context]);
+        if (hook_signal_send (signal_name,
+                              WEECHAT_HOOK_SIGNAL_STRING,
+                              gui_key_combo_buffer) == WEECHAT_RC_OK_EAT)
+        {
+            gui_key_combo_buffer[0] = '\0';
+            return 0;
+        }
+    }
+
+    if (gui_key_is_complete (gui_key_combo_buffer))
+        gui_key_combo_buffer[0] = '\0';
 
     /*
      * if this is first key and not found (even partial) => return 1
@@ -1847,22 +1896,22 @@ gui_key_hdata_key_cb (void *data, const char *hdata_name)
                       "gui_keys%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &gui_keys[i]);
+            hdata_new_list(hdata, str_list, &gui_keys[i], 0);
             snprintf (str_list, sizeof (str_list),
                       "last_gui_key%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &last_gui_key[i]);
+            hdata_new_list(hdata, str_list, &last_gui_key[i], 0);
             snprintf (str_list, sizeof (str_list),
                       "gui_default_keys%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &gui_default_keys[i]);
+            hdata_new_list(hdata, str_list, &gui_default_keys[i], 0);
             snprintf (str_list, sizeof (str_list),
                       "last_gui_default_key%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &last_gui_default_key[i]);
+            hdata_new_list(hdata, str_list, &last_gui_default_key[i], 0);
         }
     }
     return hdata;
@@ -1895,7 +1944,10 @@ gui_key_add_to_infolist (struct t_infolist *infolist, struct t_gui_key *key)
     if (expanded_key)
     {
         if (!infolist_new_var_string (ptr_item, "key", expanded_key))
+        {
+            free (expanded_key);
             return 0;
+        }
         free (expanded_key);
     }
     if (!infolist_new_var_integer (ptr_item, "area_type1", key->area_type[0]))
