@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,15 +17,13 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_GUI_KEY_H
-#define __WEECHAT_GUI_KEY_H 1
+#ifndef WEECHAT_GUI_KEY_H
+#define WEECHAT_GUI_KEY_H 1
 
 struct t_hashtable;
 
 #define GUI_KEY_BUFFER_BLOCK_SIZE 256
 
-#define GUI_KEY_GRAB_DELAY_DEFAULT 500
-
 #define GUI_KEY_BRACKETED_PASTE_START  "\x1B[200~"
 #define GUI_KEY_BRACKETED_PASTE_END    "\x1B[201~"
 #define GUI_KEY_BRACKETED_PASTE_LENGTH 6
@@ -58,7 +56,7 @@ struct t_gui_key
     int area_type[2];               /* type of areas (for cursor/mouse)     */
     char *area_name[2];             /* name of areas (for cursor/mouse)     */
     char *area_key;                 /* key after area (after ":")           */
-    char *command;                  /* associated command (may be NULL)     */
+    char *command;                  /* associated command                   */
     int score;                      /* score, for sorting keys              */
     struct t_gui_key *prev_key;     /* link to previous key                 */
     struct t_gui_key *next_key;     /* link to next key                     */
@@ -141,4 +139,4 @@ extern void gui_key_print_log (struct t_gui_buffer *buffer);
 
 extern void gui_key_default_bindings ();
 
-#endif /* __WEECHAT_GUI_KEY_H */
+#endif /* WEECHAT_GUI_KEY_H */
