@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,8 +17,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_FIFO_H
-#define __WEECHAT_FIFO_H 1
+#ifndef WEECHAT_FIFO_H
+#define WEECHAT_FIFO_H 1
 
 #define weechat_plugin weechat_fifo_plugin
 #define FIFO_PLUGIN_NAME "fifo"
@@ -26,4 +26,4 @@
 extern struct t_weechat_plugin *weechat_fifo_plugin;
 extern char *fifo_filename;
 
-#endif /* __WEECHAT_FIFO_H */
+#endif /* WEECHAT_FIFO_H */
