@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,8 +17,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_IRC_CHANNEL_H
-#define __WEECHAT_IRC_CHANNEL_H 1
+#ifndef WEECHAT_IRC_CHANNEL_H
+#define WEECHAT_IRC_CHANNEL_H 1
 
 #define IRC_CHANNEL_DEFAULT_CHANTYPES "#&+!"
 
@@ -50,7 +50,7 @@ struct t_irc_channel
     struct t_hashtable *join_msg_received; /* messages received after join: */
                                        /* 353=names, 366=names count,       */
                                        /* 332/333=topic, 329=creation date  */
-    int checking_away;                 /* = 1 if checking away with WHO cmd */
+    int checking_whox;                 /* = 1 if checking WHOX              */
     char *away_message;                /* to display away only once in pv   */
     int has_quit_server;               /* =1 if nick has quit (pv only), to */
                                        /* display message when he's back    */
@@ -78,6 +78,11 @@ struct t_irc_channel
 
 extern int irc_channel_valid (struct t_irc_server *server,
                               struct t_irc_channel *channel);
+extern struct t_gui_buffer *irc_channel_create_buffer (struct t_irc_server *server,
+                                                       int channel_type,
+                                                       const char *channel_name,
+                                                       int switch_to_channel,
+                                                       int auto_switch);
 extern struct t_irc_channel *irc_channel_new (struct t_irc_server *server,
                                               int channel_type,
                                               const char *channel_name,
@@ -85,6 +90,7 @@ extern struct t_irc_channel *irc_channel_new (struct t_irc_server *server,
                                               int auto_switch);
 extern void irc_channel_add_nicklist_groups (struct t_irc_server *server,
                                              struct t_irc_channel *channel);
+extern void irc_channel_set_buffer_title (struct t_irc_channel *channel);
 extern void irc_channel_set_topic (struct t_irc_channel *channel,
                                    const char *topic);
 extern void irc_channel_set_modes (struct t_irc_channel *channel,
@@ -96,9 +102,13 @@ extern struct t_irc_channel *irc_channel_search (struct t_irc_server *server,
                                                  const char *channel_name);
 extern int irc_channel_is_channel (struct t_irc_server *server,
                                    const char *string);
+extern const char *irc_channel_get_auto_chantype (struct t_irc_server *server,
+                                                  const char *channel_name);
+extern void irc_channel_remove_account (struct t_irc_server *server,
+                                        struct t_irc_channel *channel);
 extern void irc_channel_remove_away (struct t_irc_server *server,
                                      struct t_irc_channel *channel);
-extern void irc_channel_check_away (struct t_irc_server *server,
+extern void irc_channel_check_whox (struct t_irc_server *server,
                                     struct t_irc_channel *channel);
 extern void irc_channel_set_away (struct t_irc_server *server,
                                   struct t_irc_channel *channel,
@@ -150,4 +160,4 @@ extern int irc_channel_add_to_infolist (struct t_infolist *infolist,
                                         struct t_irc_channel *channel);
 extern void irc_channel_print_log (struct t_irc_channel *channel);
 
-#endif /* __WEECHAT_IRC_CHANNEL_H */
+#endif /* WEECHAT_IRC_CHANNEL_H */
