@@ -1,7 +1,7 @@
 /*
  * irc-message.c - functions for IRC messages
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -30,21 +30,45 @@
 
 
 /*
- * Parses an IRC message and returns pointers to:
- *   - tags
- *   - message without tags
- *   - host
- *   - command
- *   - channel
- *   - target nick
- *   - arguments (if any)
+ * Parses an IRC message and returns:
+ *   - tags (string)
+ *   - message without tags (string)
+ *   - nick (string)
+ *   - host (string)
+ *   - command (string)
+ *   - channel (string)
+ *   - arguments (string)
+ *   - text (string)
+ *   - pos_command (integer: command index in message)
+ *   - pos_arguments (integer: arguments index in message)
+ *   - pos_channel (integer: channel index in message)
+ *   - pos_text (integer: text index in message)
+ *
+ * Example:
+ *   @time=2015-06-27T16:40:35.000Z :nick!user@host PRIVMSG #weechat :hello!
+ *
+ * Result:
+ *               tags: "time=2015-06-27T16:40:35.000Z"
+ *   msg_without_tags: ":nick!user@host PRIVMSG #weechat :hello!"
+ *               nick: "nick"
+ *               host: "nick!user@host"
+ *            command: "PRIVMSG"
+ *            channel: "#weechat"
+ *          arguments: "#weechat :hello!"
+ *               text: "hello!"
+ *        pos_command: 47
+ *      pos_arguments: 55
+ *        pos_channel: 55
+ *           pos_text: 65
  */
 
 void
 irc_message_parse (struct t_irc_server *server, const char *message,
                    char **tags, char **message_without_tags, char **nick,
                    char **host, char **command, char **channel,
-                   char **arguments)
+                   char **arguments, char **text,
+                   int *pos_command, int *pos_arguments, int *pos_channel,
+                   int *pos_text)
 {
     const char *ptr_message, *pos, *pos2, *pos3, *pos4;
 
@@ -62,6 +86,16 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         *channel = NULL;
     if (arguments)
         *arguments = NULL;
+    if (text)
+        *text = NULL;
+    if (pos_command)
+        *pos_command = -1;
+    if (pos_arguments)
+        *pos_arguments = -1;
+    if (pos_channel)
+        *pos_channel = -1;
+    if (pos_text)
+        *pos_text = -1;
 
     if (!message)
         return;
@@ -71,7 +105,7 @@ irc_message_parse (struct t_irc_server *server, const char *message,
     /*
      * we will use this message as example:
      *
-     *   @tags :FlashCode!n=flash@host.com PRIVMSG #channel :hello!
+     *   @time=2015-06-27T16:40:35.000Z :nick!user@host PRIVMSG #weechat :hello!
      */
 
     if (ptr_message[0] == '@')
@@ -85,7 +119,10 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         if (pos)
         {
             if (tags)
-                *tags = weechat_strndup (message + 1, pos - (message + 1));
+            {
+                *tags = weechat_strndup (ptr_message + 1,
+                                         pos - (ptr_message + 1));
+            }
             ptr_message = pos + 1;
             while (ptr_message[0] == ' ')
             {
@@ -97,12 +134,16 @@ irc_message_parse (struct t_irc_server *server, const char *message,
     if (message_without_tags)
         *message_without_tags = strdup (ptr_message);
 
-    /* now we have: ptr_message --> ":FlashCode!n=flash@host.com PRIVMSG #channel :hello!" */
+    /* now we have: ptr_message --> ":nick!user@host PRIVMSG #weechat :hello!" */
     if (ptr_message[0] == ':')
     {
         /* read host/nick */
+        pos3 = strchr (ptr_message, '@');
         pos2 = strchr (ptr_message, '!');
         pos = strchr (ptr_message, ' ');
+        /* if the prefix doesn't contain a '!', split the nick at '@' */
+        if (!pos2 || (pos && pos2 > pos))
+            pos2 = pos3;
         if (pos2 && (!pos || pos > pos2))
         {
             if (nick)
@@ -123,9 +164,15 @@ irc_message_parse (struct t_irc_server *server, const char *message,
                 ptr_message++;
             }
         }
+        else
+        {
+            if (host)
+                *host = strdup (ptr_message + 1);
+            ptr_message += strlen (ptr_message);
+        }
     }
 
-    /* now we have: ptr_message --> "PRIVMSG #channel :hello!" */
+    /* now we have: ptr_message --> "PRIVMSG #weechat :hello!" */
     if (ptr_message[0])
     {
         pos = strchr (ptr_message, ' ');
@@ -133,15 +180,32 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         {
             if (command)
                 *command = weechat_strndup (ptr_message, pos - ptr_message);
+            if (pos_command)
+                *pos_command = ptr_message - message;
             pos++;
             while (pos[0] == ' ')
             {
                 pos++;
             }
-            /* now we have: pos --> "#channel :hello!" */
+            /* now we have: pos --> "#weechat :hello!" */
             if (arguments)
                 *arguments = strdup (pos);
-            if (pos[0] != ':')
+            if (pos_arguments)
+                *pos_arguments = pos - message;
+            if ((pos[0] == ':')
+                && ((strncmp (ptr_message, "JOIN ", 5) == 0)
+                    || (strncmp (ptr_message, "PART ", 5) == 0)))
+            {
+                pos++;
+            }
+            if (pos[0] == ':')
+            {
+                if (text)
+                    *text = strdup (pos + 1);
+                if (pos_text)
+                    *pos_text = pos - message + 1;
+            }
+            else
             {
                 if (irc_channel_is_channel (server, pos))
                 {
@@ -153,6 +217,21 @@ irc_message_parse (struct t_irc_server *server, const char *message,
                         else
                             *channel = strdup (pos);
                     }
+                    if (pos_channel)
+                        *pos_channel = pos - message;
+                    if (pos2)
+                    {
+                        while (pos2[0] == ' ')
+                        {
+                            pos2++;
+                        }
+                        if (pos2[0] == ':')
+                            pos2++;
+                        if (text)
+                            *text = strdup (pos2);
+                        if (pos_text)
+                            *pos_text = pos2 - message;
+                    }
                 }
                 else
                 {
@@ -182,10 +261,29 @@ irc_message_parse (struct t_irc_server *server, const char *message,
                                 else
                                     *channel = strdup (pos2);
                             }
+                            if (pos_channel)
+                                *pos_channel = pos2 - message;
+                            if (pos4)
+                            {
+                                while (pos4[0] == ' ')
+                                {
+                                    pos4++;
+                                }
+                                if (pos4[0] == ':')
+                                    pos4++;
+                                if (text)
+                                    *text = strdup (pos4);
+                                if (pos_text)
+                                    *pos_text = pos4 - message;
+                            }
                         }
-                        else if (channel && !*channel)
+                        else if ((channel && !*channel)
+                                 || (pos_channel && (*pos_channel < 0)))
                         {
-                            *channel = weechat_strndup (pos, pos3 - pos);
+                            if (channel)
+                                *channel = weechat_strndup (pos, pos3 - pos);
+                            if (pos_channel)
+                                *pos_channel = pos - message;
                         }
                     }
                 }
@@ -195,6 +293,8 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         {
             if (command)
                 *command = strdup (ptr_message);
+            if (pos_command)
+                *pos_command = ptr_message - message;
         }
     }
 }
@@ -208,6 +308,11 @@ irc_message_parse (struct t_irc_server *server, const char *message,
  *   - command
  *   - channel
  *   - arguments
+ *   - text
+ *   - pos_command
+ *   - pos_arguments
+ *   - pos_channel
+ *   - pos_text
  *
  * Note: hashtable must be freed after use.
  */
@@ -217,12 +322,14 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
                                 const char *message)
 {
     char *tags,*message_without_tags, *nick, *host, *command, *channel;
-    char *arguments;
+    char *arguments, *text, str_pos[32];
     char empty_str[1] = { '\0' };
+    int pos_command, pos_arguments, pos_channel, pos_text;
     struct t_hashtable *hashtable;
 
     irc_message_parse (server, message, &tags, &message_without_tags, &nick,
-                       &host, &command, &channel, &arguments);
+                       &host, &command, &channel, &arguments, &text,
+                       &pos_command, &pos_arguments, &pos_channel, &pos_text);
 
     hashtable = weechat_hashtable_new (32,
                                        WEECHAT_HASHTABLE_STRING,
@@ -232,13 +339,30 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
     if (!hashtable)
         return NULL;
 
-    weechat_hashtable_set (hashtable, "tags", (tags) ? tags : empty_str);
-    weechat_hashtable_set (hashtable, "message_without_tags", (message_without_tags) ? message_without_tags : empty_str);
-    weechat_hashtable_set (hashtable, "nick", (nick) ? nick : empty_str);
-    weechat_hashtable_set (hashtable, "host", (host) ? host : empty_str);
-    weechat_hashtable_set (hashtable, "command", (command) ? command : empty_str);
-    weechat_hashtable_set (hashtable, "channel", (channel) ? channel : empty_str);
-    weechat_hashtable_set (hashtable, "arguments", (arguments) ? arguments : empty_str);
+    weechat_hashtable_set (hashtable, "tags",
+                           (tags) ? tags : empty_str);
+    weechat_hashtable_set (hashtable, "message_without_tags",
+                           (message_without_tags) ? message_without_tags : empty_str);
+    weechat_hashtable_set (hashtable, "nick",
+                           (nick) ? nick : empty_str);
+    weechat_hashtable_set (hashtable, "host",
+                           (host) ? host : empty_str);
+    weechat_hashtable_set (hashtable, "command",
+                           (command) ? command : empty_str);
+    weechat_hashtable_set (hashtable, "channel",
+                           (channel) ? channel : empty_str);
+    weechat_hashtable_set (hashtable, "arguments",
+                           (arguments) ? arguments : empty_str);
+    weechat_hashtable_set (hashtable, "text",
+                           (text) ? text : empty_str);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_command);
+    weechat_hashtable_set (hashtable, "pos_command", str_pos);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_arguments);
+    weechat_hashtable_set (hashtable, "pos_arguments", str_pos);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_channel);
+    weechat_hashtable_set (hashtable, "pos_channel", str_pos);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_text);
+    weechat_hashtable_set (hashtable, "pos_text", str_pos);
 
     if (tags)
         free (tags);
@@ -254,10 +378,49 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
         free (channel);
     if (arguments)
         free (arguments);
+    if (text)
+        free (text);
 
     return hashtable;
 }
 
+/*
+ * Encodes/decodes an IRC message using a charset.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+irc_message_convert_charset (const char *message, int pos_start,
+                             const char *modifier, const char *modifier_data)
+{
+    char *text, *msg_result;
+    int length;
+
+    text = weechat_hook_modifier_exec (modifier, modifier_data,
+                                       message + pos_start);
+    if (!text)
+        return NULL;
+
+    length = pos_start + strlen (text) + 1;
+    msg_result = malloc (length);
+    if (msg_result)
+    {
+        msg_result[0] = '\0';
+        if (pos_start > 0)
+        {
+            memcpy (msg_result, message, pos_start);
+            msg_result[pos_start] = '\0';
+        }
+        strcat (msg_result, text);
+    }
+
+    free (text);
+
+    return msg_result;
+}
+
+
 /*
  * Gets nick from host in an IRC message.
  */
@@ -872,11 +1035,11 @@ irc_message_split (struct t_irc_server *server, const char *message)
          * ISON :nick1 nick2 nick3
          * WALLOPS :some text here
          */
-        split_ok = irc_message_split_string (hashtable, tags, host, command,
-                                             NULL, ":",
-                                             (argv_eol[index_args][0] == ':') ?
-                                             argv_eol[index_args] + 1 : argv_eol[index_args],
-                                             NULL, ' ', max_length_host);
+        split_ok = irc_message_split_string (
+            hashtable, tags, host, command, NULL, ":",
+            (argv_eol[index_args][0] == ':') ?
+            argv_eol[index_args] + 1 : argv_eol[index_args],
+            NULL, ' ', max_length_host);
     }
     else if (weechat_strcasecmp (command, "monitor") == 0)
     {
@@ -889,18 +1052,17 @@ irc_message_split (struct t_irc_server *server, const char *message)
         {
             snprintf (monitor_action, sizeof (monitor_action),
                       "%c ", argv_eol[index_args][0]);
-            split_ok = irc_message_split_string (hashtable, tags, host, command,
-                                                 NULL, monitor_action,
-                                                 argv_eol[index_args] + 2,
-                                                 NULL, ',', max_length_host);
+            split_ok = irc_message_split_string (
+                hashtable, tags, host, command, NULL, monitor_action,
+                argv_eol[index_args] + 2, NULL, ',', max_length_host);
         }
         else
         {
-            split_ok = irc_message_split_string (hashtable, tags, host, command,
-                                                 NULL, ":",
-                                                 (argv_eol[index_args][0] == ':') ?
-                                                 argv_eol[index_args] + 1 : argv_eol[index_args],
-                                                 NULL, ',', max_length_host);
+            split_ok = irc_message_split_string (
+                hashtable, tags, host, command, NULL, ":",
+                (argv_eol[index_args][0] == ':') ?
+                argv_eol[index_args] + 1 : argv_eol[index_args],
+                NULL, ',', max_length_host);
         }
     }
     else if (weechat_strcasecmp (command, "join") == 0)
@@ -922,12 +1084,11 @@ irc_message_split (struct t_irc_server *server, const char *message)
          */
         if (index_args + 1 <= argc - 1)
         {
-            split_ok = irc_message_split_privmsg_notice (hashtable, tags, host,
-                                                         command,
-                                                         argv[index_args],
-                                                         (argv_eol[index_args + 1][0] == ':') ?
-                                                         argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1],
-                                                         max_length_host);
+            split_ok = irc_message_split_privmsg_notice (
+                hashtable, tags, host, command, argv[index_args],
+                (argv_eol[index_args + 1][0] == ':') ?
+                argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1],
+                max_length_host);
         }
     }
     else if (weechat_strcasecmp (command, "005") == 0)
@@ -935,10 +1096,10 @@ irc_message_split (struct t_irc_server *server, const char *message)
         /* :server 005 nick MODES=4 CHANLIMIT=#:20 NICKLEN=16 USERLEN=10 ... */
         if (index_args + 1 <= argc - 1)
         {
-            split_ok = irc_message_split_005 (hashtable, tags, host, command,
-                                              argv[index_args],
-                                              (argv_eol[index_args + 1][0] == ':') ?
-                                              argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1]);
+            split_ok = irc_message_split_005 (
+                hashtable, tags, host, command, argv[index_args],
+                (argv_eol[index_args + 1][0] == ':') ?
+                argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1]);
         }
     }
     else if (weechat_strcasecmp (command, "353") == 0)
@@ -953,11 +1114,11 @@ irc_message_split (struct t_irc_server *server, const char *message)
             {
                 snprintf (target, sizeof (target), "%s %s",
                           argv[index_args], argv[index_args + 1]);
-                split_ok = irc_message_split_string (hashtable, tags, host,
-                                                     command, target, ":",
-                                                     (argv_eol[index_args + 2][0] == ':') ?
-                                                     argv_eol[index_args + 2] + 1 : argv_eol[index_args + 2],
-                                                     NULL, ' ', -1);
+                split_ok = irc_message_split_string (
+                    hashtable, tags, host, command, target, ":",
+                    (argv_eol[index_args + 2][0] == ':') ?
+                    argv_eol[index_args + 2] + 1 : argv_eol[index_args + 2],
+                    NULL, ' ', -1);
             }
             else
             {
@@ -966,11 +1127,11 @@ irc_message_split (struct t_irc_server *server, const char *message)
                     snprintf (target, sizeof (target), "%s %s %s",
                               argv[index_args], argv[index_args + 1],
                               argv[index_args + 2]);
-                    split_ok = irc_message_split_string (hashtable, tags, host,
-                                                         command, target, ":",
-                                                         (argv_eol[index_args + 3][0] == ':') ?
-                                                         argv_eol[index_args + 3] + 1 : argv_eol[index_args + 3],
-                                                         NULL, ' ', -1);
+                    split_ok = irc_message_split_string (
+                        hashtable, tags, host, command, target, ":",
+                        (argv_eol[index_args + 3][0] == ':') ?
+                        argv_eol[index_args + 3] + 1 : argv_eol[index_args + 3],
+                        NULL, ' ', -1);
                 }
             }
         }
