@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,8 +17,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_IRC_MODE_H
-#define __WEECHAT_IRC_MODE_H 1
+#ifndef WEECHAT_IRC_MODE_H
+#define WEECHAT_IRC_MODE_H 1
 
 struct t_irc_server;
 struct t_irc_channel;
@@ -29,4 +29,4 @@ extern int irc_mode_channel_set (struct t_irc_server *server,
 extern void irc_mode_user_set (struct t_irc_server *server, const char *modes,
                                int reset_modes);
 
-#endif /* __WEECHAT_IRC_MODE_H */
+#endif /* WEECHAT_IRC_MODE_H */
