@@ -1,7 +1,7 @@
 /*
  * irc-nick.c - nick management for IRC plugin
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -95,6 +95,8 @@ irc_nick_is_nick (const char *string)
 /*
  * Duplicates a nick and stops at first char in list (using option
  * irc.look.nick_color_stop_chars).
+ *
+ * Note: result must be freed after use.
  */
 
 char *
@@ -624,7 +626,7 @@ irc_nick_nicklist_set_color_all ()
 struct t_irc_nick *
 irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
               const char *nickname, const char *host, const char *prefixes,
-              int away)
+              int away, const char *account)
 {
     struct t_irc_nick *new_nick, *ptr_nick;
     int length;
@@ -639,15 +641,15 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     ptr_nick = irc_nick_search (server, channel, nickname);
     if (ptr_nick)
     {
-        /* save away status from existing nick (before removing it) */
-        away = ptr_nick->away;
-
         /* remove old nick from nicklist */
         irc_nick_nicklist_remove (server, channel, ptr_nick);
 
         /* update nick */
         irc_nick_set_prefixes (server, ptr_nick, prefixes);
         ptr_nick->away = away;
+        if (ptr_nick->account)
+            free (ptr_nick->account);
+        ptr_nick->account = (account) ? strdup (account) : NULL;
 
         /* add new nick in nicklist */
         irc_nick_nicklist_add (server, channel, ptr_nick);
@@ -662,8 +664,22 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     /* initialize new nick */
     new_nick->name = strdup (nickname);
     new_nick->host = (host) ? strdup (host) : NULL;
+    new_nick->account = (account) ? strdup (account) : NULL;
     length = strlen (irc_server_get_prefix_chars (server));
     new_nick->prefixes = malloc (length + 1);
+    if (!new_nick->name || !new_nick->prefixes)
+    {
+        if (new_nick->name)
+            free (new_nick->name);
+        if (new_nick->host)
+            free (new_nick->host);
+        if (new_nick->account)
+            free (new_nick->account);
+        if (new_nick->prefixes)
+            free (new_nick->prefixes);
+        free (new_nick);
+        return NULL;
+    }
     if (new_nick->prefixes)
     {
         memset (new_nick->prefixes, ' ', length);
@@ -799,6 +815,8 @@ irc_nick_free (struct t_irc_server *server, struct t_irc_channel *channel,
         free (nick->host);
     if (nick->prefixes)
         free (nick->prefixes);
+    if (nick->account)
+        free (nick->account);
     if (nick->color)
         free (nick->color);
 
@@ -903,6 +921,7 @@ irc_nick_set_away (struct t_irc_server *server, struct t_irc_channel *channel,
                    struct t_irc_nick *nick, int is_away)
 {
     if (!is_away
+        || server->cap_away_notify
         || ((IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK) > 0)
             && ((IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS) == 0)
                 || (channel->nicks_count <= IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS)))))
@@ -947,12 +966,13 @@ irc_nick_mode_for_display (struct t_irc_server *server, struct t_irc_nick *nick,
             {
                 str_prefix[0] = '\0';
             }
-            str_prefix_color = weechat_color (irc_nick_get_prefix_color_name (server,
-                                                                              nick->prefix[0]));
+            str_prefix_color = weechat_color (
+                irc_nick_get_prefix_color_name (server, nick->prefix[0]));
         }
         else
         {
-            str_prefix[0] = (prefix && weechat_config_boolean (irc_config_look_nick_mode_empty)) ?
+            str_prefix[0] = (prefix
+                             && weechat_config_boolean (irc_config_look_nick_mode_empty)) ?
                 ' ' : '\0';
             str_prefix_color = IRC_COLOR_RESET;
         }
@@ -992,10 +1012,15 @@ irc_nick_as_prefix (struct t_irc_server *server, struct t_irc_nick *nick,
  */
 
 const char *
-irc_nick_color_for_message (struct t_irc_server *server,
-                            struct t_irc_nick *nick,
-                            const char *nickname)
+irc_nick_color_for_msg (struct t_irc_server *server, int server_message,
+                        struct t_irc_nick *nick, const char *nickname)
 {
+    if (server_message
+        && !weechat_config_boolean (irc_config_look_color_nicks_in_server_messages))
+    {
+        return IRC_COLOR_CHAT_NICK;
+    }
+
     if (nick)
         return nick->color;
 
@@ -1012,21 +1037,6 @@ irc_nick_color_for_message (struct t_irc_server *server,
     return IRC_COLOR_CHAT_NICK;
 }
 
-/*
- * Returns WeeChat color code for a nick (used in a server message).
- */
-
-const char *
-irc_nick_color_for_server_message (struct t_irc_server *server,
-                                   struct t_irc_nick *nick,
-                                   const char *nickname)
-{
-    if (!weechat_config_boolean(irc_config_look_color_nicks_in_server_messages))
-        return IRC_COLOR_CHAT_NICK;
-
-    return irc_nick_color_for_message (server, nick, nickname);
-}
-
 /*
  * Returns string with color of nick for private.
  */
@@ -1055,7 +1065,7 @@ char *
 irc_nick_default_ban_mask (struct t_irc_nick *nick)
 {
     const char *ptr_ban_mask;
-    char *pos_hostname, user[128], *res, *temp;
+    char *pos_hostname, user[128], ident[128], *res, *temp;
 
     if (!nick)
         return NULL;
@@ -1072,6 +1082,7 @@ irc_nick_default_ban_mask (struct t_irc_nick *nick)
 
     strncpy (user, nick->host, pos_hostname - nick->host);
     user[pos_hostname - nick->host] = '\0';
+    strcpy (ident, (user[0] != '~') ? user : "*");
     pos_hostname++;
 
     /* replace nick */
@@ -1087,6 +1098,13 @@ irc_nick_default_ban_mask (struct t_irc_nick *nick)
         return NULL;
     res = temp;
 
+    /* replace ident */
+    temp = weechat_string_replace (res, "$ident", ident);
+    free (res);
+    if (!temp)
+        return NULL;
+    res = temp;
+
     /* replace hostname */
     temp = weechat_string_replace (res, "$host", pos_hostname);
     free (res);
@@ -1118,6 +1136,7 @@ irc_nick_hdata_nick_cb (void *data, const char *hdata_name)
         WEECHAT_HDATA_VAR(struct t_irc_nick, prefixes, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, prefix, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, away, INTEGER, 0, NULL, NULL);
+        WEECHAT_HDATA_VAR(struct t_irc_nick, account, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, color, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, prev_nick, POINTER, 0, NULL, hdata_name);
         WEECHAT_HDATA_VAR(struct t_irc_nick, next_nick, POINTER, 0, NULL, hdata_name);
@@ -1156,6 +1175,8 @@ irc_nick_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "away", nick->away))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "account", nick->account))
+        return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "color", nick->color))
         return 0;
 
@@ -1175,6 +1196,7 @@ irc_nick_print_log (struct t_irc_nick *nick)
     weechat_log_printf ("         prefixes . . . : '%s'",  nick->prefixes);
     weechat_log_printf ("         prefix . . . . : '%s'",  nick->prefix);
     weechat_log_printf ("         away . . . . . : %d",    nick->away);
+    weechat_log_printf ("         account. . . . : '%s'",  nick->account);
     weechat_log_printf ("         color. . . . . : '%s'",  nick->color);
     weechat_log_printf ("         prev_nick. . . : 0x%lx", nick->prev_nick);
     weechat_log_printf ("         next_nick. . . : 0x%lx", nick->next_nick);
