@@ -1,7 +1,7 @@
 /*
  * irc-notify.c - notify lists for IRC plugin
  *
- * Copyright (C) 2010-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -434,8 +434,8 @@ void
 irc_notify_free (struct t_irc_server *server, struct t_irc_notify *notify,
                  int remove_monitor)
 {
-    weechat_hook_signal_send ("irc_notify_removing",
-                              WEECHAT_HOOK_SIGNAL_POINTER, notify);
+    (void) weechat_hook_signal_send ("irc_notify_removing",
+                                     WEECHAT_HOOK_SIGNAL_POINTER, notify);
 
     /* free data */
     if (notify->nick)
@@ -468,8 +468,8 @@ irc_notify_free (struct t_irc_server *server, struct t_irc_notify *notify,
     if (server->notify_count > 0)
         server->notify_count--;
 
-    weechat_hook_signal_send ("irc_notify_removed",
-                              WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    (void) weechat_hook_signal_send ("irc_notify_removed",
+                                     WEECHAT_HOOK_SIGNAL_STRING, NULL);
 }
 
 /*
@@ -505,40 +505,40 @@ irc_notify_display (struct t_irc_server *server, struct t_gui_buffer *buffer,
     if ((notify->is_on_server < 0)
         || (!notify->is_on_server && !notify->away_message))
     {
-        weechat_printf (buffer,
-                        "  %s%s%s @ %s%s%s: %s%s",
-                        irc_nick_color_for_server_message (server, NULL,
-                                                           notify->nick),
-                        notify->nick,
-                        IRC_COLOR_RESET,
-                        IRC_COLOR_CHAT_SERVER,
-                        notify->server->name,
-                        IRC_COLOR_RESET,
-                        (notify->is_on_server < 0) ? "" : IRC_COLOR_MESSAGE_QUIT,
-                        (notify->is_on_server < 0) ?
-                        /* TRANSLATORS: "unknown" is the status for /notify when ison answer has not been received (check pending) */
-                        _("unknown") :
-                        _("offline"));
+        weechat_printf (
+            buffer,
+            "  %s%s%s @ %s%s%s: %s%s",
+            irc_nick_color_for_msg (server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            IRC_COLOR_CHAT_SERVER,
+            notify->server->name,
+            IRC_COLOR_RESET,
+            (notify->is_on_server < 0) ? "" : IRC_COLOR_MESSAGE_QUIT,
+            (notify->is_on_server < 0) ?
+            /* TRANSLATORS: "unknown" is the status for /notify when ison answer has not been received (check pending) */
+            _("unknown") :
+            _("offline"));
     }
     else
     {
-        weechat_printf (buffer,
-                        "  %s%s%s @ %s%s%s: %s%s %s%s%s%s%s%s",
-                        irc_nick_color_for_server_message (server, NULL,
-                                                           notify->nick),
-                        notify->nick,
-                        IRC_COLOR_RESET,
-                        IRC_COLOR_CHAT_SERVER,
-                        notify->server->name,
-                        IRC_COLOR_RESET,
-                        IRC_COLOR_MESSAGE_JOIN,
-                        _("online"),
-                        IRC_COLOR_RESET,
-                        (notify->away_message) ? " (" : "",
-                        (notify->away_message) ? _("away") : "",
-                        (notify->away_message) ? ": \"" : "",
-                        (notify->away_message) ? notify->away_message : "",
-                        (notify->away_message) ? "\")" : "");
+        weechat_printf (
+            buffer,
+            "  %s%s%s @ %s%s%s: %s%s %s%s%s%s%s%s",
+            irc_nick_color_for_msg (server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            IRC_COLOR_CHAT_SERVER,
+            notify->server->name,
+            IRC_COLOR_RESET,
+            IRC_COLOR_MESSAGE_JOIN,
+            _("online"),
+            IRC_COLOR_RESET,
+            (notify->away_message) ? " (" : "",
+            (notify->away_message) ? _("away") : "",
+            (notify->away_message) ? ": \"" : "",
+            (notify->away_message) ? notify->away_message : "",
+            (notify->away_message) ? "\")" : "");
     }
 }
 
@@ -650,7 +650,7 @@ irc_notify_send_signal (struct t_irc_notify *notify,
                   (away_message && away_message[0]) ? away_message : "");
     }
 
-    weechat_hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING, data);
+    (void) weechat_hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING, data);
 
     if (data)
         free (data);
@@ -672,29 +672,28 @@ irc_notify_set_is_on_server (struct t_irc_notify *notify, const char *host,
     if (notify->is_on_server == is_on_server)
         return;
 
-    weechat_printf_tags (notify->server->buffer,
-                         irc_notify_get_tags (irc_config_look_notify_tags_ison,
-                                              (is_on_server) ? "join" : "quit",
-                                              notify->nick),
-                         (notify->is_on_server < 0) ?
-                         ((is_on_server) ?
-                          _("%snotify: %s%s%s%s%s%s%s%s%s is connected") :
-                          _("%snotify: %s%s%s%s%s%s%s%s%s is offline")) :
-                         ((is_on_server) ?
-                          _("%snotify: %s%s%s%s%s%s%s%s%s has joined") :
-                          _("%snotify: %s%s%s%s%s%s%s%s%s has quit")),
-                         weechat_prefix ("network"),
-                         irc_nick_color_for_server_message (notify->server,
-                                                            NULL,
-                                                            notify->nick),
-                         notify->nick,
-                         (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
-                         (host && host[0]) ? " (" : "",
-                         (host && host[0]) ? IRC_COLOR_CHAT_HOST : "",
-                         (host && host[0]) ? host : "",
-                         (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
-                         (host && host[0]) ? ")" : "",
-                         (is_on_server) ? IRC_COLOR_MESSAGE_JOIN : IRC_COLOR_MESSAGE_QUIT);
+    weechat_printf_tags (
+        notify->server->buffer,
+        irc_notify_get_tags (irc_config_look_notify_tags_ison,
+                             (is_on_server) ? "join" : "quit",
+                             notify->nick),
+        (notify->is_on_server < 0) ?
+        ((is_on_server) ?
+         _("%snotify: %s%s%s%s%s%s%s%s%s is connected") :
+         _("%snotify: %s%s%s%s%s%s%s%s%s is offline")) :
+        ((is_on_server) ?
+         _("%snotify: %s%s%s%s%s%s%s%s%s has connected") :
+         _("%snotify: %s%s%s%s%s%s%s%s%s has quit")),
+        weechat_prefix ("network"),
+        irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+        notify->nick,
+        (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
+        (host && host[0]) ? " (" : "",
+        (host && host[0]) ? IRC_COLOR_CHAT_HOST : "",
+        (host && host[0]) ? host : "",
+        (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
+        (host && host[0]) ? ")" : "",
+        (is_on_server) ? IRC_COLOR_MESSAGE_JOIN : IRC_COLOR_MESSAGE_QUIT);
     irc_notify_send_signal (notify, (is_on_server) ? "join" : "quit", NULL);
 
     notify->is_on_server = is_on_server;
@@ -720,49 +719,43 @@ irc_notify_set_away_message (struct t_irc_notify *notify,
 
     if (!notify->away_message && away_message)
     {
-        weechat_printf_tags (notify->server->buffer,
-                             irc_notify_get_tags (irc_config_look_notify_tags_whois,
-                                                  "away",
-                                                  notify->nick),
-                             _("%snotify: %s%s%s is now away: \"%s\""),
-                             weechat_prefix ("network"),
-                             irc_nick_color_for_server_message (notify->server,
-                                                                NULL,
-                                                                notify->nick),
-                             notify->nick,
-                             IRC_COLOR_RESET,
-                             away_message);
+        weechat_printf_tags (
+            notify->server->buffer,
+            irc_notify_get_tags (
+                irc_config_look_notify_tags_whois, "away", notify->nick),
+            _("%snotify: %s%s%s is now away: \"%s\""),
+            weechat_prefix ("network"),
+            irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            away_message);
         irc_notify_send_signal (notify, "away", away_message);
     }
     else if (notify->away_message && !away_message)
     {
-        weechat_printf_tags (notify->server->buffer,
-                             irc_notify_get_tags (irc_config_look_notify_tags_whois,
-                                                  "back",
-                                                  notify->nick),
-                             _("%snotify: %s%s%s is back"),
-                             weechat_prefix ("network"),
-                             irc_nick_color_for_server_message (notify->server,
-                                                                NULL,
-                                                                notify->nick),
-                             notify->nick,
-                             IRC_COLOR_RESET);
+        weechat_printf_tags (
+            notify->server->buffer,
+            irc_notify_get_tags (
+                irc_config_look_notify_tags_whois, "back", notify->nick),
+            _("%snotify: %s%s%s is back"),
+            weechat_prefix ("network"),
+            irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET);
         irc_notify_send_signal (notify, "back", NULL);
     }
     else if (notify->away_message && away_message)
     {
-        weechat_printf_tags (notify->server->buffer,
-                             irc_notify_get_tags (irc_config_look_notify_tags_whois,
-                                                  "still_away",
-                                                  notify->nick),
-                             _("%snotify: %s%s%s is still away: \"%s\""),
-                             weechat_prefix ("network"),
-                             irc_nick_color_for_server_message (notify->server,
-                                                                NULL,
-                                                                notify->nick),
-                             notify->nick,
-                             IRC_COLOR_RESET,
-                             away_message);
+        weechat_printf_tags (
+            notify->server->buffer,
+            irc_notify_get_tags (
+                irc_config_look_notify_tags_whois, "still_away", notify->nick),
+            _("%snotify: %s%s%s is still away: \"%s\""),
+            weechat_prefix ("network"),
+            irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            away_message);
         irc_notify_send_signal (notify, "still_away", away_message);
     }
 
@@ -842,7 +835,8 @@ irc_notify_hsignal_cb (void *data, const char *signal,
             for (i = 0; i < num_messages; i++)
             {
                 irc_message_parse (ptr_server, messages[i], NULL, NULL, NULL,
-                                   NULL, NULL, NULL, &arguments);
+                                   NULL, NULL, NULL, &arguments, NULL, NULL,
+                                   NULL, NULL, NULL);
                 if (arguments)
                 {
                     pos = strchr (arguments, ' ');
@@ -924,7 +918,8 @@ irc_notify_hsignal_cb (void *data, const char *signal,
                 for (i = 0; i < num_messages; i++)
                 {
                     irc_message_parse (ptr_server, messages[0], NULL, NULL,
-                                       NULL, NULL, &irc_cmd, NULL, &arguments);
+                                       NULL, NULL, &irc_cmd, NULL, &arguments,
+                                       NULL, NULL, NULL, NULL, NULL);
                     if (irc_cmd && arguments)
                     {
                         if (strcmp (irc_cmd, "401") == 0)
