@@ -1,7 +1,7 @@
 /*
  * irc-upgrade.c - save/restore IRC plugin data when upgrading WeeChat
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -245,6 +245,7 @@ irc_upgrade_set_buffer_callbacks ()
 {
     struct t_infolist *infolist;
     struct t_gui_buffer *ptr_buffer;
+    struct t_irc_server *ptr_server;
     const char *type;
 
     infolist = weechat_infolist_get ("buffer", NULL, NULL);
@@ -260,8 +261,17 @@ irc_upgrade_set_buffer_callbacks ()
                 type = weechat_buffer_get_string (ptr_buffer, "localvar_type");
                 if (type && (strcmp (type, "channel") == 0))
                 {
+                    ptr_server = irc_server_search (
+                        weechat_buffer_get_string (ptr_buffer,
+                                                   "localvar_server"));
                     weechat_buffer_set_pointer (ptr_buffer, "nickcmp_callback",
                                                 &irc_buffer_nickcmp_cb);
+                    if (ptr_server)
+                    {
+                        weechat_buffer_set_pointer (ptr_buffer,
+                                                    "nickcmp_callback_data",
+                                                    ptr_server);
+                    }
                 }
                 if (strcmp (weechat_infolist_string (infolist, "name"),
                             IRC_RAW_BUFFER_NAME) == 0)
@@ -361,6 +371,8 @@ irc_upgrade_read_cb (void *data,
                     str = weechat_infolist_string (infolist, "nick_modes");
                     if (str)
                         irc_upgrade_current_server->nick_modes = strdup (str);
+                    irc_upgrade_current_server->cap_away_notify = weechat_infolist_integer (infolist, "cap_away_notify");
+                    irc_upgrade_current_server->cap_account_notify = weechat_infolist_integer (infolist, "cap_account_notify");
                     str = weechat_infolist_string (infolist, "isupport");
                     if (str)
                         irc_upgrade_current_server->isupport = strdup (str);
@@ -432,6 +444,7 @@ irc_upgrade_read_cb (void *data,
                         irc_upgrade_current_server->away_message = strdup (str);
                     irc_upgrade_current_server->away_time = weechat_infolist_time (infolist, "away_time");
                     irc_upgrade_current_server->lag = weechat_infolist_integer (infolist, "lag");
+                    irc_upgrade_current_server->lag_displayed = weechat_infolist_integer (infolist, "lag_displayed");
                     buf = weechat_infolist_buffer (infolist, "lag_check_time", &size);
                     if (buf)
                         memcpy (&(irc_upgrade_current_server->lag_check_time), buf, size);
@@ -476,7 +489,7 @@ irc_upgrade_read_cb (void *data,
                                 weechat_string_free_split (items);
                             }
                         }
-                        irc_upgrade_current_channel->checking_away = weechat_infolist_integer (infolist, "checking_away");
+                        irc_upgrade_current_channel->checking_whox = weechat_infolist_integer (infolist, "checking_whox");
                         str = weechat_infolist_string (infolist, "away_message");
                         if (str)
                             irc_upgrade_current_channel->away_message = strdup (str);
@@ -560,7 +573,8 @@ irc_upgrade_read_cb (void *data,
                                              weechat_infolist_string (infolist, "name"),
                                              weechat_infolist_string (infolist, "host"),
                                              weechat_infolist_string (infolist, "prefixes"),
-                                             weechat_infolist_integer (infolist, "away"));
+                                             weechat_infolist_integer (infolist, "away"),
+                                             weechat_infolist_string (infolist, "account"));
                     if (ptr_nick)
                     {
                         /*
@@ -719,7 +733,10 @@ irc_upgrade_load ()
     irc_upgrade_set_buffer_callbacks ();
 
     upgrade_file = weechat_upgrade_new (IRC_UPGRADE_FILENAME, 0);
+    if (!upgrade_file)
+        return 0;
     rc = weechat_upgrade_read (upgrade_file, &irc_upgrade_read_cb, NULL);
+    weechat_upgrade_close (upgrade_file);
 
     return rc;
 }
