@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,8 +17,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_LOGGER_CONFIG_H
-#define __WEECHAT_LOGGER_CONFIG_H 1
+#ifndef WEECHAT_LOGGER_CONFIG_H
+#define WEECHAT_LOGGER_CONFIG_H 1
 
 #define LOGGER_CONFIG_NAME "logger"
 
@@ -47,4 +47,4 @@ extern int logger_config_read ();
 extern int logger_config_write ();
 extern void logger_config_free ();
 
-#endif /* __WEECHAT_LOGGER_CONFIG_H */
+#endif /* WEECHAT_LOGGER_CONFIG_H */
