@@ -1,7 +1,7 @@
 /*
  * logger-tail.c - return last lines of a file
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -41,13 +41,13 @@
  * Searches for last EOL in a string.
  */
 
-char *
+const char *
 logger_tail_last_eol (const char *string_start, const char *string_ptr)
 {
     while (string_ptr >= string_start)
     {
         if ((string_ptr[0] == '\n') || (string_ptr[0] == '\r'))
-            return (char *)string_ptr;
+            return string_ptr;
         string_ptr--;
     }
 
@@ -58,7 +58,7 @@ logger_tail_last_eol (const char *string_start, const char *string_ptr)
 /*
  * Returns last lines of a file.
  *
- * Note: result must be freed with function "logger_tail_free".
+ * Note: result must be freed after use with function logger_tail_free().
  */
 
 struct t_logger_line *
@@ -101,6 +101,8 @@ logger_tail_file (const char *filename, int n_lines)
         bytes_read = read (fd, buf, to_read);
         if (bytes_read <= 0)
         {
+            if (part_of_line)
+                free (part_of_line);
             logger_tail_free (ptr_line);
             close (fd);
             return NULL;
@@ -109,7 +111,7 @@ logger_tail_file (const char *filename, int n_lines)
         ptr_buf = buf + bytes_read - 1;
         while (ptr_buf && (ptr_buf >= buf))
         {
-            pos_eol = logger_tail_last_eol (buf, ptr_buf);
+            pos_eol = (char *)logger_tail_last_eol (buf, ptr_buf);
             if ((pos_eol && (pos_eol[1] || part_of_line)) || (file_pos == 0))
             {
                 /* use data and part_of_line (if existing) to build a new line */
@@ -140,6 +142,7 @@ logger_tail_file (const char *filename, int n_lines)
                         if (!new_line->data)
                         {
                             free (part_of_line);
+                            free (new_line);
                             logger_tail_free (ptr_line);
                             close (fd);
                             return NULL;
