@@ -32,7 +32,7 @@
 #define PyBytes_Check PyString_Check
 #define PyBytes_FromString PyString_FromString
 #define PyUnicode_FromString PyString_FromString
-#endif
+#endif /* PY_VERSION_HEX < 0x02060000 */
 
 #if PY_MAJOR_VERSION >= 3
 /* check of integer with Python >= 3.x */
@@ -40,7 +40,7 @@
 #else
 /* check of integer with Python <= 2.x */
 #define PY_INTEGER_CHECK(x) (PyInt_Check(x) || PyLong_Check(x))
-#endif
+#endif /* PY_MAJOR_VERSION >= 3 */
 
 extern struct t_weechat_plugin *weechat_python_plugin;
 
