@@ -1,7 +1,7 @@
 /*
  * relay-irc.c - IRC protocol for relay to client: IRC proxy/bouncer
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -438,7 +438,7 @@ relay_irc_signal_irc_outtags_cb (void *data, const char *signal,
             && relay_irc_command_relayed (irc_command))
         {
             /* get host for nick (it is self nick) */
-            snprintf (str_infolist_args, sizeof (str_infolist_args) - 1,
+            snprintf (str_infolist_args, sizeof (str_infolist_args),
                       "%s,%s,%s",
                       client->protocol_args,
                       irc_channel,
@@ -571,13 +571,13 @@ relay_irc_hsignal_irc_redir_cb (void *data, const char *signal,
  *   - date
  *   - nick
  *   - nick1 and nick2 (old and new nick for irc "nick" command)
- *   - host (without colors)
+ *   - host
  *   - message (without colors).
  *
  * Arguments hdata_line_data and line_data must be non NULL, the other arguments
  * can be NULL.
  *
- * Note: tags, host and message (if given and filled) must be freed after use.
+ * Note: tags and message (if given and filled) must be freed after use.
  */
 
 void
@@ -586,13 +586,13 @@ relay_irc_get_line_info (struct t_relay_client *client,
                          struct t_hdata *hdata_line_data, void *line_data,
                          int *irc_command, int *irc_action, time_t *date,
                          const char **nick, const char **nick1,
-                         const char **nick2, char **tags, char **host,
-                         char **message)
+                         const char **nick2, const char **host,
+                         char **tags, char **message)
 {
     int i, num_tags, command, action, all_tags, length;
-    char str_tag[256], *pos, *pos2, *message_no_color, str_time[256];
+    char str_tag[256], *pos, *message_no_color, str_time[256];
     const char *ptr_tag, *ptr_message, *ptr_nick, *ptr_nick1, *ptr_nick2;
-    const char *localvar_nick, *time_format;
+    const char *ptr_host, *localvar_nick, *time_format;
     time_t msg_date;
     struct tm *tm;
 
@@ -608,10 +608,10 @@ relay_irc_get_line_info (struct t_relay_client *client,
         *nick1 = NULL;
     if (nick2)
         *nick2 = NULL;
-    if (tags)
-        *tags = NULL;
     if (host)
         *host = NULL;
+    if (tags)
+        *tags = NULL;
     if (message)
         *message = NULL;
 
@@ -629,6 +629,7 @@ relay_irc_get_line_info (struct t_relay_client *client,
     ptr_nick = NULL;
     ptr_nick1 = NULL;
     ptr_nick2 = NULL;
+    ptr_host = NULL;
     all_tags = weechat_hashtable_has_key (relay_config_hashtable_irc_backlog_tags,
                                           "*");
     for (i = 0; i < num_tags; i++)
@@ -645,6 +646,8 @@ relay_irc_get_line_info (struct t_relay_client *client,
                 ptr_nick1 = ptr_tag + 10;
             else if (strncmp (ptr_tag, "irc_nick2_", 10) == 0)
                 ptr_nick2 = ptr_tag + 10;
+            else if (strncmp (ptr_tag, "host_", 5) == 0)
+                ptr_host = ptr_tag + 5;
             else if ((command < 0)
                      && (all_tags
                          || (weechat_hashtable_has_key (relay_config_hashtable_irc_backlog_tags,
@@ -665,9 +668,8 @@ relay_irc_get_line_info (struct t_relay_client *client,
     {
         localvar_nick = weechat_buffer_get_string (buffer, "localvar_nick");
         if (localvar_nick && localvar_nick[0]
-            && (strcmp (ptr_nick, localvar_nick) == 0))
+            && ptr_nick && (strcmp (ptr_nick, localvar_nick) == 0))
         {
-            command = -1;
             return;
         }
     }
@@ -685,8 +687,11 @@ relay_irc_get_line_info (struct t_relay_client *client,
         *nick1 = ptr_nick1;
     if (nick2)
         *nick2 = ptr_nick2;
+    if (host)
+        *host = ptr_host;
     message_no_color = (ptr_message) ?
         weechat_string_remove_color (ptr_message, NULL) : NULL;
+
     if ((command == RELAY_IRC_CMD_PRIVMSG) && message && message_no_color)
     {
         pos = message_no_color;
@@ -721,25 +726,7 @@ relay_irc_get_line_info (struct t_relay_client *client,
         else
             *message = strdup (pos);
     }
-    switch (command)
-    {
-        case RELAY_IRC_CMD_JOIN:
-        case RELAY_IRC_CMD_PART:
-        case RELAY_IRC_CMD_QUIT:
-            if (host && message && message_no_color)
-            {
-                pos = strstr (message_no_color, " (");
-                if (pos)
-                {
-                    pos2 = strchr (pos, ')');
-                    if (pos2)
-                        *host = weechat_strndup (pos + 2, pos2 - pos - 2);
-                }
-            }
-            break;
-        default:
-            break;
-    }
+
     /* if server capability "server-time" is enabled, add an irc tag with time */
     if (tags
         && (RELAY_IRC_DATA(client, server_capabilities) & (1 << RELAY_IRC_CAPAB_SERVER_TIME)))
@@ -766,8 +753,8 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
     struct t_relay_server *ptr_server;
     void *ptr_own_lines, *ptr_line, *ptr_line_data;
     void *ptr_hdata_line, *ptr_hdata_line_data;
-    char *tags, *host, *message;
-    const char *ptr_nick, *ptr_nick1, *ptr_nick2;
+    char *tags, *message;
+    const char *ptr_nick, *ptr_nick1, *ptr_nick2, *ptr_host;
     int irc_command, irc_action, count, max_number, max_minutes;
     time_t date_min, date_min2, date;
 
@@ -826,8 +813,8 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
                                      NULL, /* nick */
                                      NULL, /* nick1 */
                                      NULL, /* nick2 */
-                                     NULL, /* tags */
                                      NULL, /* host */
+                                     NULL, /* tags */
                                      NULL); /* message */
             if (irc_command >= 0)
             {
@@ -873,8 +860,8 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
                                      &ptr_nick,
                                      &ptr_nick1,
                                      &ptr_nick2,
+                                     &ptr_host,
                                      &tags,
-                                     &host,
                                      &message);
             switch (irc_command)
             {
@@ -883,8 +870,8 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
                                      "%s:%s%s%s JOIN :%s",
                                      (tags) ? tags : "",
                                      ptr_nick,
-                                     (host) ? "!" : "",
-                                     (host) ? host : "",
+                                     (ptr_host) ? "!" : "",
+                                     (ptr_host) ? ptr_host : "",
                                      channel);
                     break;
                 case RELAY_IRC_CMD_PART:
@@ -892,16 +879,16 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
                                      "%s:%s%s%s PART %s",
                                      (tags) ? tags : "",
                                      ptr_nick,
-                                     (host) ? "!" : "",
-                                     (host) ? host : "",
+                                     (ptr_host) ? "!" : "",
+                                     (ptr_host) ? ptr_host : "",
                                      channel);
                 case RELAY_IRC_CMD_QUIT:
                     relay_irc_sendf (client,
                                      "%s:%s%s%s QUIT",
                                      (tags) ? tags : "",
                                      ptr_nick,
-                                     (host) ? "!" : "",
-                                     (host) ? host : "");
+                                     (ptr_host) ? "!" : "",
+                                     (ptr_host) ? ptr_host : "");
                     break;
                 case RELAY_IRC_CMD_NICK:
                     if (ptr_nick1 && ptr_nick2)
@@ -917,9 +904,11 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
                     if (ptr_nick && message)
                     {
                         relay_irc_sendf (client,
-                                         "%s:%s PRIVMSG %s :%s%s%s",
+                                         "%s:%s%s%s PRIVMSG %s :%s%s%s",
                                          (tags) ? tags : "",
                                          ptr_nick,
+                                         (ptr_host) ? "!" : "",
+                                         (ptr_host) ? ptr_host : "",
                                          channel,
                                          (irc_action) ? "\01ACTION " : "",
                                          message,
@@ -932,8 +921,6 @@ relay_irc_send_channel_backlog (struct t_relay_client *client,
             }
             if (tags)
                 free (tags);
-            if (host)
-                free (host);
             if (message)
                 free (message);
         }
@@ -1146,9 +1133,8 @@ relay_irc_input_send (struct t_relay_client *client, const char *irc_channel,
                             RELAY_PLUGIN_NAME, buf);
         }
 
-        weechat_hook_signal_send ("irc_input_send",
-                                  WEECHAT_HOOK_SIGNAL_STRING,
-                                  buf);
+        (void) weechat_hook_signal_send ("irc_input_send",
+                                         WEECHAT_HOOK_SIGNAL_STRING, buf);
         free (buf);
     }
     free (vbuffer);
@@ -1209,28 +1195,6 @@ relay_irc_hook_signals (struct t_relay_client *client)
                               client);
 }
 
-/*
- * Timer called to hooks signals and send joins for all channels to client.
- */
-
-int
-relay_irc_timer_hooks_and_joins_cb (void *data, int remaining_calls)
-{
-    /* make C compiler happy */
-    (void) remaining_calls;
-
-    if (relay_client_valid (data))
-    {
-        /* hook signals */
-        relay_irc_hook_signals (data);
-
-        /* send JOIN for all channels on server to client */
-        relay_irc_send_join_channels (data);
-    }
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Processes the "CAP" irc command (received from client)
  */
@@ -1258,6 +1222,8 @@ relay_irc_recv_command_capab (struct t_relay_client *client,
                              (RELAY_IRC_DATA(client, nick)) ? RELAY_IRC_DATA(client, nick) : "nick",
                              str_capab);
         }
+        if (!RELAY_IRC_DATA(client, connected))
+            RELAY_IRC_DATA(client, cap_ls_received) = 1;
     }
     else if (weechat_strncasecmp (arguments, "req ", 4) == 0)
     {
@@ -1300,6 +1266,11 @@ relay_irc_recv_command_capab (struct t_relay_client *client,
             weechat_string_free_split (capabs);
         }
     }
+    else if (weechat_strcasecmp (arguments, "end") == 0)
+    {
+        if (!RELAY_IRC_DATA(client, connected))
+            RELAY_IRC_DATA(client, cap_end_received) = 1;
+    }
 }
 
 /*
@@ -1387,7 +1358,12 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
                     if (password)
                     {
                         if (strcmp (password, pos_password) == 0)
+                        {
                             RELAY_IRC_DATA(client, password_ok) = 1;
+                            weechat_hook_signal_send ("relay_client_auth_ok",
+                                                      WEECHAT_HOOK_SIGNAL_POINTER,
+                                                      client);
+                        }
                         free (password);
                     }
                 }
@@ -1441,7 +1417,9 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
             }
         }
         if (RELAY_IRC_DATA(client, nick)
-            && RELAY_IRC_DATA(client, user_received))
+            && RELAY_IRC_DATA(client, user_received)
+            && (!RELAY_IRC_DATA(client, cap_ls_received)
+                || RELAY_IRC_DATA(client, cap_end_received)))
         {
             /* disconnect client if password was not received or wrong */
             if (!RELAY_IRC_DATA(client, password_ok))
@@ -1540,17 +1518,11 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
                              RELAY_IRC_DATA(client, address),
                              RELAY_IRC_DATA(client, nick));
 
-            /*
-             * hook a timer which will hook signals and send JOIN for all
-             * channels to client (a timer is needed because we may not have
-             * received the server capabilities yet, like server-time)
-             */
-            if (RELAY_IRC_DATA(client, hook_timer_signals_joins))
-                weechat_unhook (RELAY_IRC_DATA(client, hook_timer_signals_joins));
-            RELAY_IRC_DATA(client, hook_timer_signals_joins) =
-                weechat_hook_timer (500, 0, 1,
-                                    &relay_irc_timer_hooks_and_joins_cb,
-                                    client);
+            /* hook signals */
+            relay_irc_hook_signals (client);
+
+            /* send JOIN for all channels on server to client */
+            relay_irc_send_join_channels (client);
         }
     }
     else
@@ -1630,58 +1602,58 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
                 weechat_hashtable_set (hash_redirect, "signal", str_signal);
                 if (weechat_strcasecmp (irc_command, "mode") == 0)
                 {
-                    if (irc_argc == 1)
-                        redirect_msg = 1;
-                    weechat_hashtable_set (hash_redirect, "pattern",
-                                           "mode_channel");
                     if (irc_argc > 0)
                     {
+                        if (irc_argc == 1)
+                            redirect_msg = 1;
+                        weechat_hashtable_set (hash_redirect, "pattern",
+                                               "mode_channel");
                         weechat_hashtable_set (hash_redirect, "string",
                                                irc_argv[0]);
-                    }
-                    snprintf (str_server_channel,
-                              sizeof (str_server_channel),
-                              "%s,%s",
-                              client->protocol_args,
-                              irc_argv[0]);
-                    info = weechat_info_get ("irc_is_channel",
-                                             str_server_channel);
-                    if (info && (strcmp (info, "1") == 0))
-                    {
-                        /* command "MODE #channel ..." */
-                        if (irc_argc == 2)
+                        snprintf (str_server_channel,
+                                  sizeof (str_server_channel),
+                                  "%s,%s",
+                                  client->protocol_args,
+                                  irc_argv[0]);
+                        info = weechat_info_get ("irc_is_channel",
+                                                 str_server_channel);
+                        if (info && (strcmp (info, "1") == 0))
                         {
-                            if ((strcmp (irc_argv[1], "b") == 0)
-                                || (strcmp (irc_argv[1], "+b") == 0))
+                            /* command "MODE #channel ..." */
+                            if (irc_argc == 2)
                             {
-                                redirect_msg = 1;
-                                weechat_hashtable_set (hash_redirect, "pattern",
-                                                       "mode_channel_ban");
-                            }
-                            else if ((strcmp (irc_argv[1], "e") == 0)
-                                     || (strcmp (irc_argv[1], "+e") == 0))
-                            {
-                                redirect_msg = 1;
-                                weechat_hashtable_set (hash_redirect, "pattern",
-                                                       "mode_channel_ban_exception");
-                            }
-                            else if ((strcmp (irc_argv[1], "I") == 0)
-                                     || (strcmp (irc_argv[1], "+I") == 0))
-                            {
-                                redirect_msg = 1;
-                                weechat_hashtable_set (hash_redirect, "pattern",
-                                                       "mode_channel_invite");
+                                if ((strcmp (irc_argv[1], "b") == 0)
+                                    || (strcmp (irc_argv[1], "+b") == 0))
+                                {
+                                    redirect_msg = 1;
+                                    weechat_hashtable_set (hash_redirect, "pattern",
+                                                           "mode_channel_ban");
+                                }
+                                else if ((strcmp (irc_argv[1], "e") == 0)
+                                         || (strcmp (irc_argv[1], "+e") == 0))
+                                {
+                                    redirect_msg = 1;
+                                    weechat_hashtable_set (hash_redirect, "pattern",
+                                                           "mode_channel_ban_exception");
+                                }
+                                else if ((strcmp (irc_argv[1], "I") == 0)
+                                         || (strcmp (irc_argv[1], "+I") == 0))
+                                {
+                                    redirect_msg = 1;
+                                    weechat_hashtable_set (hash_redirect, "pattern",
+                                                           "mode_channel_invite");
+                                }
                             }
                         }
-                    }
-                    else
-                    {
-                        /* command "MODE nick ..." */
-                        if (irc_argc == 1)
+                        else
                         {
-                            redirect_msg = 1;
-                            weechat_hashtable_set (hash_redirect, "pattern",
-                                                   "mode_user");
+                            /* command "MODE nick ..." */
+                            if (irc_argc == 1)
+                            {
+                                redirect_msg = 1;
+                                weechat_hashtable_set (hash_redirect, "pattern",
+                                                       "mode_user");
+                            }
                         }
                     }
                 }
@@ -1729,8 +1701,8 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
                  */
                 if (redirect_msg)
                 {
-                    weechat_hook_hsignal_send ("irc_redirect_command",
-                                               hash_redirect);
+                    (void) weechat_hook_hsignal_send ("irc_redirect_command",
+                                                      hash_redirect);
                 }
                 weechat_hashtable_free (hash_redirect);
             }
@@ -1757,11 +1729,6 @@ relay_irc_close_connection (struct t_relay_client *client)
 {
     RELAY_IRC_DATA(client, connected) = 0;
 
-    if (RELAY_IRC_DATA(client, hook_timer_signals_joins))
-    {
-        weechat_unhook (RELAY_IRC_DATA(client, hook_timer_signals_joins));
-        RELAY_IRC_DATA(client, hook_timer_signals_joins) = NULL;
-    }
     if (RELAY_IRC_DATA(client, hook_signal_irc_in2))
     {
         weechat_unhook (RELAY_IRC_DATA(client, hook_signal_irc_in2));
@@ -1804,9 +1771,10 @@ relay_irc_alloc (struct t_relay_client *client)
         RELAY_IRC_DATA(client, password_ok) = (password && password[0]) ? 0 : 1;
         RELAY_IRC_DATA(client, nick) = NULL;
         RELAY_IRC_DATA(client, user_received) = 0;
+        RELAY_IRC_DATA(client, cap_ls_received) = 0;
+        RELAY_IRC_DATA(client, cap_end_received) = 0;
         RELAY_IRC_DATA(client, connected) = 0;
         RELAY_IRC_DATA(client, server_capabilities) = 0;
-        RELAY_IRC_DATA(client, hook_timer_signals_joins) = NULL;
         RELAY_IRC_DATA(client, hook_signal_irc_in2) = NULL;
         RELAY_IRC_DATA(client, hook_signal_irc_outtags) = NULL;
         RELAY_IRC_DATA(client, hook_signal_irc_disc) = NULL;
@@ -1839,9 +1807,10 @@ relay_irc_alloc_with_infolist (struct t_relay_client *client,
         else
             RELAY_IRC_DATA(client, nick) = NULL;
         RELAY_IRC_DATA(client, user_received) = weechat_infolist_integer (infolist, "user_received");
+        RELAY_IRC_DATA(client, cap_ls_received) = weechat_infolist_integer (infolist, "cap_ls_received");
+        RELAY_IRC_DATA(client, cap_end_received) = weechat_infolist_integer (infolist, "cap_end_received");
         RELAY_IRC_DATA(client, connected) = weechat_infolist_integer (infolist, "connected");
         RELAY_IRC_DATA(client, server_capabilities) = weechat_infolist_integer (infolist, "server_capabilities");
-        RELAY_IRC_DATA(client, hook_timer_signals_joins) = NULL;
         if (RELAY_IRC_DATA(client, connected))
         {
             relay_irc_hook_signals (client);
@@ -1869,8 +1838,6 @@ relay_irc_free (struct t_relay_client *client)
             free (RELAY_IRC_DATA(client, address));
         if (RELAY_IRC_DATA(client, nick))
             free (RELAY_IRC_DATA(client, nick));
-        if (RELAY_IRC_DATA(client, hook_timer_signals_joins))
-            weechat_unhook (RELAY_IRC_DATA(client, hook_timer_signals_joins));
         if (RELAY_IRC_DATA(client, hook_signal_irc_in2))
             weechat_unhook (RELAY_IRC_DATA(client, hook_signal_irc_in2));
         if (RELAY_IRC_DATA(client, hook_signal_irc_outtags))
@@ -1909,12 +1876,14 @@ relay_irc_add_to_infolist (struct t_infolist_item *item,
         return 0;
     if (!weechat_infolist_new_var_integer (item, "user_received", RELAY_IRC_DATA(client, user_received)))
         return 0;
+    if (!weechat_infolist_new_var_integer (item, "cap_ls_received", RELAY_IRC_DATA(client, cap_ls_received)))
+        return 0;
+    if (!weechat_infolist_new_var_integer (item, "cap_end_received", RELAY_IRC_DATA(client, cap_end_received)))
+        return 0;
     if (!weechat_infolist_new_var_integer (item, "connected", RELAY_IRC_DATA(client, connected)))
         return 0;
     if (!weechat_infolist_new_var_integer (item, "server_capabilities", RELAY_IRC_DATA(client, server_capabilities)))
         return 0;
-    if (!weechat_infolist_new_var_pointer (item, "hook_timer_signals_joins", RELAY_IRC_DATA(client, hook_timer_signals_joins)))
-        return 0;
     if (!weechat_infolist_new_var_pointer (item, "hook_signal_irc_in2", RELAY_IRC_DATA(client, hook_signal_irc_in2)))
         return 0;
     if (!weechat_infolist_new_var_pointer (item, "hook_signal_irc_outtags", RELAY_IRC_DATA(client, hook_signal_irc_outtags)))
@@ -1940,9 +1909,10 @@ relay_irc_print_log (struct t_relay_client *client)
         weechat_log_printf ("    password_ok . . . . . . : %d",    RELAY_IRC_DATA(client, password_ok));
         weechat_log_printf ("    nick. . . . . . . . . . : '%s'",  RELAY_IRC_DATA(client, nick));
         weechat_log_printf ("    user_received . . . . . : %d",    RELAY_IRC_DATA(client, user_received));
+        weechat_log_printf ("    cap_ls_received . . . . : %d",    RELAY_IRC_DATA(client, cap_ls_received));
+        weechat_log_printf ("    cap_end_received. . . . : %d",    RELAY_IRC_DATA(client, cap_end_received));
         weechat_log_printf ("    connected . . . . . . . : %d",    RELAY_IRC_DATA(client, connected));
         weechat_log_printf ("    server_capabilities . . : %d",    RELAY_IRC_DATA(client, server_capabilities));
-        weechat_log_printf ("    hook_timer_signals_joins: 0x%lx", RELAY_IRC_DATA(client, hook_timer_signals_joins));
         weechat_log_printf ("    hook_signal_irc_in2 . . : 0x%lx", RELAY_IRC_DATA(client, hook_signal_irc_in2));
         weechat_log_printf ("    hook_signal_irc_outtags : 0x%lx", RELAY_IRC_DATA(client, hook_signal_irc_outtags));
         weechat_log_printf ("    hook_signal_irc_disc. . : 0x%lx", RELAY_IRC_DATA(client, hook_signal_irc_disc));
