@@ -1,7 +1,7 @@
 /*
  * relay-server.c - server functions for relay plugin
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -269,6 +269,12 @@ relay_server_sock_cb (void *data, int fd)
                        INET6_ADDRSTRLEN))
         {
             ptr_ip_address = ipv6_address;
+
+            if (strncmp (ptr_ip_address, "::ffff:", 7) == 0)
+            {
+                /* actually an IPv4-mapped IPv6 address, so skip "::ffff:" */
+                ptr_ip_address += 7;
+            }
         }
     }
     else
@@ -423,7 +429,7 @@ relay_server_create_socket (struct t_relay_server *server)
             return 0;
         }
     }
-#endif
+#endif /* IPV6_V6ONLY */
 
     /* set option SO_REUSEADDR to 1 */
     set = 1;
@@ -540,15 +546,7 @@ relay_server_new (const char *protocol_string, enum t_relay_protocol protocol,
         new_server->start_time = 0;
         new_server->last_client_disconnect = 0;
 
-        if (!relay_server_create_socket (new_server))
-        {
-            if (new_server->protocol_string)
-                free (new_server->protocol_string);
-            if (new_server->protocol_args)
-                free (new_server->protocol_args);
-            free (new_server);
-            return NULL;
-        }
+        relay_server_create_socket (new_server);
 
         new_server->prev_server = NULL;
         new_server->next_server = relay_servers;
