@@ -1,7 +1,7 @@
 /*
  * relay-websocket.c - websocket server functions for relay plugin (RFC 6455)
  *
- * Copyright (C) 2013-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2013-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -145,7 +145,7 @@ relay_websocket_client_handshake_valid (struct t_relay_client *client)
     value = weechat_hashtable_get (client->http_headers, "Upgrade");
     if (!value)
         return -1;
-    if (strcmp (value, "websocket") != 0)
+    if (weechat_strcasecmp (value, "websocket") != 0)
         return -1;
 
     /* check if we have header "Sec-WebSocket-Key" with non-empty value */
@@ -215,6 +215,8 @@ relay_websocket_build_handshake (struct t_relay_client *client)
     weechat_string_encode_base64 ((char *)result, length, sec_websocket_accept);
     gcry_md_close (hd);
 
+    free (key);
+
     /* build the handshake (it will be sent as-is to client) */
     snprintf (handshake, sizeof (handshake),
               "HTTP/1.1 101 Switching Protocols\r\n"
