@@ -1,7 +1,7 @@
 /*
  * script-action.c - actions on scripts
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -208,7 +208,6 @@ script_action_unload (const char *name, int quiet)
     struct t_hdata *hdata;
     void *ptr_script;
 
-    language = -1;
     pos = strrchr (name, '.');
     if (pos)
     {
@@ -314,7 +313,6 @@ script_action_reload (const char *name, int quiet)
     struct t_hdata *hdata;
     void *ptr_script;
 
-    language = -1;
     pos = strrchr (name, '.');
     if (pos)
     {
@@ -464,9 +462,9 @@ script_action_autoload (const char *name, int quiet, int autoload)
         snprintf (str_signal, sizeof (str_signal),
                   "%s_script_autoload",
                   script_language[ptr_script->language]);
-        weechat_hook_signal_send (str_signal,
-                                  WEECHAT_HOOK_SIGNAL_STRING,
-                                  filename);
+        (void) weechat_hook_signal_send (str_signal,
+                                         WEECHAT_HOOK_SIGNAL_STRING,
+                                         filename);
         free (filename);
     }
     if (!quiet)
@@ -508,20 +506,23 @@ script_action_install_process_cb (void *data, const char *command,
     int quiet, length;
     struct t_script_repo *ptr_script;
 
+    /* make C compiler happy */
+    (void) out;
+
     quiet = (data) ? 1 : 0;
 
     if (return_code >= 0)
     {
         pos = strrchr (command, '/');
 
-        if ((err && err[0]) || (out && (strncmp (out, "error:", 6) == 0)))
+        if (err && err[0])
         {
             weechat_printf (NULL,
                             _("%s%s: error downloading script \"%s\": %s"),
                             weechat_prefix ("error"),
                             SCRIPT_PLUGIN_NAME,
                             (pos) ? pos + 1 : "?",
-                            (err && err[0]) ? err : out + 6);
+                            err);
             return WEECHAT_RC_OK;
         }
 
@@ -546,9 +547,9 @@ script_action_install_process_cb (void *data, const char *command,
                         snprintf (str_signal, sizeof (str_signal),
                                   "%s_script_install",
                                   script_language[ptr_script->language]);
-                        weechat_hook_signal_send (str_signal,
-                                                  WEECHAT_HOOK_SIGNAL_STRING,
-                                                  filename2);
+                        (void) weechat_hook_signal_send (str_signal,
+                                                         WEECHAT_HOOK_SIGNAL_STRING,
+                                                         filename2);
                         free (filename2);
                     }
                     free (filename);
@@ -599,7 +600,6 @@ script_action_install (int quiet)
 {
     struct t_script_repo *ptr_script_to_install;
     char *filename, *url;
-    int length;
     struct t_hashtable *options;
 
     while (1)
@@ -637,8 +637,7 @@ script_action_install (int quiet)
                                          NULL);
         if (options)
         {
-            length = 4 + strlen (ptr_script_to_install->url) + 1;
-            url = malloc (length);
+            url = script_build_download_url (ptr_script_to_install->url);
             if (url)
             {
                 if (!weechat_config_boolean (script_config_look_quiet_actions))
@@ -648,9 +647,6 @@ script_action_install (int quiet)
                                     SCRIPT_PLUGIN_NAME,
                                     ptr_script_to_install->name_with_extension);
                 }
-
-                snprintf (url, length, "url:%s",
-                          ptr_script_to_install->url);
                 weechat_hashtable_set (options, "file_out", filename);
                 weechat_hook_process_hashtable (url, options, 30000,
                                                 &script_action_install_process_cb,
@@ -734,9 +730,9 @@ script_action_remove (const char *name, int quiet)
         snprintf (str_signal, sizeof (str_signal),
                   "%s_script_remove",
                   script_language[ptr_script->language]);
-        weechat_hook_signal_send (str_signal,
-                                  WEECHAT_HOOK_SIGNAL_STRING,
-                                  filename);
+        (void) weechat_hook_signal_send (str_signal,
+                                         WEECHAT_HOOK_SIGNAL_STRING,
+                                         filename);
         free (filename);
     }
 }
@@ -903,19 +899,20 @@ script_action_show_source_process_cb (void *data, const char *command,
 
     /* make C compiler happy */
     (void) data;
+    (void) out;
 
     if (return_code >= 0)
     {
         pos = strrchr (command, '/');
 
-        if ((err && err[0]) || (out && (strncmp (out, "error:", 6) == 0)))
+        if (err && err[0])
         {
             weechat_printf (NULL,
                             _("%s%s: error downloading script \"%s\": %s"),
                             weechat_prefix ("error"),
                             SCRIPT_PLUGIN_NAME,
                             (pos) ? pos + 1 : "?",
-                            (err && err[0]) ? err : out + 6);
+                            err);
             return WEECHAT_RC_OK;
         }
 
@@ -1029,7 +1026,6 @@ script_action_show (const char *name, int quiet)
 {
     struct t_script_repo *ptr_script;
     char *filename, *url;
-    int length;
     struct t_hashtable *options;
 
     if (name)
@@ -1067,11 +1063,9 @@ script_action_show (const char *name, int quiet)
                                                      NULL);
                     if (options)
                     {
-                        length = 4 + strlen (ptr_script->url) + 1;
-                        url = malloc (length);
+                        url = script_build_download_url (ptr_script->url);
                         if (url)
                         {
-                            snprintf (url, length, "url:%s", ptr_script->url);
                             weechat_hashtable_set (options, "file_out", filename);
                             weechat_hook_process_hashtable (url, options, 30000,
                                                             &script_action_show_source_process_cb,
@@ -1153,6 +1147,8 @@ script_action_run ()
     if (!script_actions)
         return 0;
 
+    script_get_loaded_plugins ();
+
     actions = weechat_string_split (script_actions, "\n", 0, 0, &num_actions);
     if (actions)
     {
